# -*-  coding: utf-8 -*-

# Copyright (C) 2015 ZetaOps Inc.
#
# This file is licensed under the GNU General Public License v3
# (GPLv3).  See LICENSE.txt for details.

"""HITAP Istisnai Ilgi Ekle

Hitap'a personelin Istisnai Ilgi bilgilerinin eklemesini yapar.

"""

from ulakbus.services.personel.hitap.hitap_ekle import HITAPEkle


class HizmetIstisnaiIlgiEkle(HITAPEkle):
    """
    HITAP Ekleme servisinden kalıtılmış Hizmet Istisnai Bilgi Ekleme servisi

    """
    HAS_CHANNEL = True

    def handle(self):
        """Servis çağrıldığında tetiklenen metod.

        Attributes:
            service_name (str): İlgili Hitap sorgu servisinin adı
            service_dict (dict): Request yoluyla gelen kayıtlar,
                    hizmetIstisnaiIlgiUpdate servisinin alanlarıyla eşlenmektedir.
                    Filtreden geçecek tarih alanları ve servis tarafında gerekli olan
                    alanlar listede tutulmaktadır.

        """

        self.service_name = 'hizmetIstisnaiIlgiInsert'
        self.service_dict = {
            'fields': {
                'kayitNo': self.request.payload.get('kayit_no', ''),
                'tckn': self.request.payload.get('tckn', ''),
                'istisnaiIlgiNevi': self.request.payload.get('istisnai_ilgi_nevi', ''),
                'baslamaTarihi': self.request.payload.get('baslama_tarihi', ''),
                'bitisTarihi': self.request.payload.get('bitis_tarihi', ''),
                'gunSayisi': self.request.payload.get('gun_sayisi', ''),
                'khaDurum': self.request.payload.get('kha_durum', ''),
                'kurumOnayTarihi': self.request.payload.get('kurum_onay_tarihi', '')
            },
            'date_filter': ['baslamaTarihi', 'bitisTarihi', 'kurumOnayTarihi'],
            'required_fields': ['tckn', 'istisnaiIlgiNevi', 'baslamaTarihi',
                                'bitisTarihi', 'gunSayisi', 'khaDurum', 'kurumOnayTarihi']
        }
        super(HizmetIstisnaiIlgiEkle, self).handle()



# -*-  coding: utf-8 -*-
"""
"""
# Copyright (C) 2015 ZetaOps Inc.
#
# This file is licensed under the GNU General Public License v3
# (GPLv3).  See LICENSE.txt for details.

import datetime
from ulakbus.lib.date_time_helper import zaman_araligi

__author__ = 'Ali Riza Keles'


def personel_izin_gunlerini_getir(okutman, yil, ay):
    """
    Args:
        okutman: okutman object
        yil: 2016
        ay: 7

    Returns: Seçilen yıl ve ay içinde
    okutmanın izin ve ücretsiz izinlerini
    gün şeklinde döndüren liste.

    """
    from ulakbus.models.personel import Izin, UcretsizIzin
    from ulakbus.lib.date_time_helper import yil_ve_aya_gore_ilk_ve_son_gun

    baslangic, bitis = yil_ve_aya_gore_ilk_ve_son_gun(yil, ay)
    personel_izin_list = []
    for i in range(2):
        model = Izin if i == 0 else UcretsizIzin

        for personel_izin in model.objects.filter(personel=okutman.personel,
                                                  baslangic__gte=baslangic,
                                                  bitis__lte=bitis):
            for gun in zaman_araligi(personel_izin.baslangic, personel_izin.bitis):
                    personel_izin_list.append(gun.day)

    return personel_izin_list


def gorunen_kademe_hesapla(derece, kademe):
    """
    Args:
        derece (int): personel derece
        kademe (int): personel kademe

    Returns:
        kademe (int): limite gore hesaplanmis kademe degeri
    """
    kademe_limitleri = {1: 4, 2: 6, 3: 8, 4: 9, 5: 9, 6: 9, 7: 9, 8: 9, 9: 9, 10: 9, 11: 9,
                        12: 9, 13: 9, 14: 9, 15: 9}
    try:
        kademe = kademe_limitleri[derece] if kademe > kademe_limitleri[derece] else kademe
        return kademe
    except KeyError:
        return 0


def derece_ilerlet(pkd, der, kad):
    """
    Derece 3 kademede bir artar. Eger kademe 4 gelmise, derece 1 arttirilir
    Args:
        pkd (int): personel kadro derecesi
        der (int): derece
        kad (int): kademe

    Returns:
        der, kad: ilerletilmis derece ve kademe

    """
    if pkd < der:
        if kad == 4:
            kad = 1
            der -= 1
    return der, kad


def suren_terfi_var_mi(p):
    """
    Mevcut wfler icinde arama yaparak personel hakkinda, devam eden terfi isleminin
    olup olmadigi kontrol edilir.

    Args:
        p (str): personel key

    Returns:
        Devam eden islem varsa True, yoksa False

    """

    # TODO: wf ler icinde arama yap
    return False


def terfi_tarhine_gore_personel_listesi(baslangic_tarihi=None, bitis_tarihi=None,
                                        personel_turu=None, suren_terfi_kontrol=True):
    """
    Args:
        baslangic_tarihi (date): baslangic_tarihi
        bitis_tarihi (date): bitis_tarihi
        personel_turu (str): personel turu, 1 akademik, 2 idari
        suren_terfi_kontrol (bool): personel listesi hazirlanirken suren baska terfi islemi varmi
                                    kontrolunun yapilip yapilmayacagini kontrol eder.

    Returns:
        personeller (dict): personel kademe derece bilgileri iceren sozluk

    """

    from ulakbus.models.personel import Personel

    simdi = datetime.date.today()
    baslangic_tarihi = baslangic_tarihi or simdi
    bitis_tarihi = bitis_tarihi or simdi + datetime.timedelta(days=90)

    personeller = {}

    qs = Personel.objects.filter(personel_turu=personel_turu)

    terfisi_gelen_personeller = qs.or_filter(
        ga_sonraki_terfi_tarihi__range=[baslangic_tarihi, bitis_tarihi],
        kh_sonraki_terfi_tarihi__range=[baslangic_tarihi, bitis_tarihi],
        em_sonraki_terfi_tarihi__range=[baslangic_tarihi, bitis_tarihi]
    )

    for personel in terfisi_gelen_personeller:

        suren_terfi = False
        if suren_terfi_kontrol:
            suren_terfi = suren_terfi_var_mi(personel.key)

        if not suren_terfi_kontrol or not suren_terfi:
            # personel temel bilgileri
            p_data = {"key": personel.key, "tckn": personel.tckn, "ad": personel.ad,
                      "soyad": personel.soyad, "kadro_derece": personel.kadro.derece,
                      "suren_terfi": suren_terfi}

            # personel guncel derece ve kademeleri
            p_data.update(
                {
                    "guncel_gorev_ayligi_derece": personel.gorev_ayligi_derece,
                    "guncel_gorev_ayligi_kademe": personel.gorev_ayligi_kademe,
                    "guncel_kazanilmis_hak_derece": personel.kazanilmis_hak_derece,
                    "guncel_kazanilmis_hak_kademe": personel.kazanilmis_hak_kademe,
                    "guncel_emekli_muktesebat_derece": personel.emekli_muktesebat_derece,
                    "guncel_emekli_muktesebat_kademe": personel.emekli_muktesebat_kademe
                }
            )

            # personel guncel gorunen kademeleri
            p_data.update(
                {
                    "gorunen_gorev_ayligi_kademe": personel.gorunen_gorev_ayligi_kademe,
                    "gorunen_kazanilmis_hak_kademe": personel.gorunen_kazanilmis_hak_kademe,
                    "gorunen_emekli_muktesebat_kademe": personel.gorunen_emekli_muktesebat_kademe
                }
            )

            pkd = personel.kadro.derece

            # terfi sonrasi derece ve kademeler
            p_data["terfi_sonrasi_gorev_ayligi_derece"], p_data[
                "terfi_sonrasi_gorev_ayligi_kademe"] = derece_ilerlet(
                pkd,
                personel.gorev_ayligi_derece,
                personel.gorev_ayligi_kademe + 1)

            p_data["terfi_sonrasi_kazanilmis_hak_derece"], p_data[
                "terfi_sonrasi_kazanilmis_hak_kademe"] = derece_ilerlet(
                pkd,
                personel.kazanilmis_hak_derece,
                personel.kazanilmis_hak_kademe + 1)

            p_data["terfi_sonrasi_emekli_muktesebat_derece"], p_data[
                "terfi_sonrasi_emekli_muktesebat_kademe"] = derece_ilerlet(
                pkd,
                personel.gorev_ayligi_derece,
                personel.gorev_ayligi_kademe + 1)

            # terfi sonrasi gorunen kademeler
            p_data["terfi_sonrasi_gorunen_gorev_ayligi_kademe"] = gorunen_kademe_hesapla(
                p_data["terfi_sonrasi_gorev_ayligi_derece"],
                p_data["terfi_sonrasi_gorev_ayligi_kademe"])
            p_data["terfi_sonrasi_gorunen_kazanilmis_hak_kademe"] = gorunen_kademe_hesapla(
                p_data["terfi_sonrasi_kazanilmis_hak_derece"],
                p_data["terfi_sonrasi_kazanilmis_hak_kademe"])
            p_data["terfi_sonrasi_gorunen_emekli_muktesebat_kademe"] = gorunen_kademe_hesapla(
                p_data["terfi_sonrasi_emekli_muktesebat_derece"],
                p_data["terfi_sonrasi_emekli_muktesebat_kademe"])

            personeller[personel.key] = p_data

    return personeller



# -*-  coding: utf-8 -*-
"""
"""

# Copyright (C) 2015 ZetaOps Inc.
#
# This file is licensed under the GNU General Public License v3
# (GPLv3).  See LICENSE.txt for details.
#

from datetime import time
from zengine.views.crud import CrudView
from zengine.forms import JsonForm, fields
from zengine.lib.translation import gettext as _, format_time, gettext_lazy
from ulakbus.models.ders_sinav_programi import OgElemaniZamanPlani, ZamanCetveli, DerslikZamanPlani, ZamanDilimleri
from ulakbus.models import Room, DerslikSinavPlani


class OnayForm(JsonForm):
    onayla = fields.Button(gettext_lazy(u'Onayla'), cmd='onay')
    reddet = fields.Button(gettext_lazy(u'Reddet'), cmd='reddet')


class ZamanTablosu(CrudView):
    """
    .. code-block:: python
        # default_request:
        {
            'callbackID': string,
            'client_cmd':['string',],
            'forms': {},
            'meta': {},
            'reload_cmd': string,
            'reply_timestamp': float
            'token': string
        },

        # default_response
        {
            'callbackID': string,
            'data': {
                'form': {},
                'cmd': string,
                'token': string,
                'wf': string,
        }

    """
    def ogretim_gorevlisi_sec(self):
        """
        Switch lecturer or select default one
        .. code-block:: python
            # request:
            {
                'secili_og_elemani':{
                    'key': string   #  personel key,
                    'cmd': 'personel_sec'}
            }

            # response:
            {
                'ders_saati_durum': Boolean
            }

        """
        # default select first person
        if self.current.task_data['cmd'] == 'personel_sec':
            zaman_dilimleri = ZamanDilimleri.objects.filter(birim=self.current.role.unit)
            ogretim_elemani = OgElemaniZamanPlani.objects.get(self.current.task_data['ogretim_elemani_key'])
            zaman_cetveli = ZamanCetveli.objects.filter(ogretim_elemani_zaman_plani=ogretim_elemani, durum=3)

            toplam_ders_saat = 0
            for zd in zaman_dilimleri:
                toplam_ders_saat += (zd.zaman_dilimi_suresi * 5)

            time = 0
            for zc in zaman_cetveli:
                if zc.gun not in [6, 7]:
                    time += zc.zaman_dilimi.zaman_dilimi_suresi
            oe_ders_saati = int(ogretim_elemani.toplam_ders_saati + (ogretim_elemani.toplam_ders_saati/2))
            if time <= (toplam_ders_saat - oe_ders_saati):
                self.current.task_data['ogretim_elemani_key'] = self.current.input['secili_og_elemani']['key']
                self.current.output['ders_saati_durum'] = True
            else:
                self.current.output['ders_saati_durum'] = False

        elif self.current.task_data['cmd'] == 'hayir':
            pass
        else:
            self.current.task_data['ogretim_elemani_key'] = \
                (OgElemaniZamanPlani.objects.filter(birim=self.current.role.unit)[0]).key

    def derslik_sec_kontrol(self, cmd='', obj=None):
        """
        Switch room or select default one

        .. code-block:: python
            # request:
            {
                'secili_derslik':{
                    'key': string   #  derslik key,
                    'cmd': 'derslik_degistir'}
            }

        """
        try:
            if self.current.task_data['cmd'] == cmd:
                self.current.task_data['room_key'] = self.current.input['secili_derslik']['key']
            elif self.current.task_data['cmd'] == 'hayir':
                pass
            else:
                self.current.task_data['rooms'] = list(set(obj.objects.filter(
                                                        birim=self.current.role.unit).values_list('derslik_id')))
                self.current.task_data['room_key'] = self.current.task_data['rooms'][0]
        except:
            self.current.task_data['rooms'] = list(set(obj.objects.filter(
                                                        birim=self.current.role.unit).values_list('derslik_id')))
            self.current.task_data['room_key'] = self.current.task_data['rooms'][0]

    def derslik_sec(self):
        self.derslik_sec_kontrol('derslik_degistir', DerslikZamanPlani)

    def sinav_derslik_sec(self):
        self.derslik_sec_kontrol('derslik_degistir', DerslikSinavPlani)

    def ogretim_elemani_listele(self):
        oe_zaman_plani = OgElemaniZamanPlani.objects.filter(birim=self.current.role.unit)
        ogretim_elemanlari = list()
        for oe in oe_zaman_plani:
            ogretim_elemani = dict()
            ogretim_elemani['name'] = oe.okutman.ad + ' ' + oe.okutman.soyad
            ogretim_elemani['key'] = oe.key
            ogretim_elemanlari.append(ogretim_elemani)

        return ogretim_elemanlari

    def sinav_derslik_listele(self):
        rooms = self.current.task_data['rooms']
        return self.derslikler(rooms)

    def derslik_listele(self):
        rooms = self.current.task_data['rooms']
        return self.derslikler(rooms)

    def derslikler(self, rooms):
        derslikler_list = list()
        for room in rooms:
            r = Room.objects.get(room)
            derslik = dict()
            derslik['name'] = r.name
            derslik['key'] = r.key
            derslikler_list.append(derslik)
        return derslikler_list

    def listele(self):
        """
        show & edit timetable for room

        .. code-block:: python
            # response:
            {
                'derslik_zaman_tablosu': {
                    'derslik_key': string   # derslik key
                    'name': string,     # room name,
                    'kapasite': int,   # capacity,
                    'zaman_plani': [{
                        'key': string,     # derslik zaman plani key
                        'saat': string,  # 10:00-12:00,
                        'gun': int,     # 1 = pazartesi,
                        'durum': int    # 2 = Bolume Ait,
                        }],
                    'derslikler': [{
                        'name': string,
                        'key': string}]}
            }
        """

        if 'red_aciklamasi' in self.current.task_data:
            self.current.output['msgbox'] = {"type": "warning",
                                             "title": _(u"Talebiniz Bölüm Başkanı Tarafından Reddedildi"),
                                             "msg": self.current.task_data['red_aciklamasi']}
        _form = JsonForm()
        _form.gonder = fields.Button(_(u'Onaya Gönder'), cmd='gonder')
        self.form_out(_form)

    def ogretim_elemani_zt(self):

        self.current.ogretim_elemani = OgElemaniZamanPlani.objects.get(self.current.task_data['ogretim_elemani_key'])
        zaman_cetveli = sorted(ZamanCetveli.objects.filter(ogretim_elemani_zaman_plani=self.current.ogretim_elemani),
                               key=lambda z: (z.zaman_dilimi, z.gun))

        uygunluk_durumu = list()

        for zc in zaman_cetveli:
            durum = dict()
            durum['key'] = zc.key
            durum['saat'] = zc.zaman_dilimi.baslama_saat + ':' + zc.zaman_dilimi.baslama_dakika + '-' + \
                            zc.zaman_dilimi.bitis_saat + ':' + zc.zaman_dilimi.bitis_dakika
            durum['gun'] = zc.gun
            durum['durum'] = zc.durum
            uygunluk_durumu.append(durum)

        item = {'avatar': self.current.ogretim_elemani.okutman.personel.user.get_avatar_url(),
                'oe_key': self.current.ogretim_elemani.key,
                'name': self.current.ogretim_elemani.okutman.ad + ' ' + self.current.ogretim_elemani.okutman.soyad,
                'toplam_ders_saati': self.current.ogretim_elemani.toplam_ders_saati,
                'uygunluk_durumu': uygunluk_durumu,
                'ogretim_elemanlari': self.ogretim_elemani_listele()}

        self.output['ogretim_elemani_zt'] = item
        self.listele()

    def derslik_zt(self):
        self.current.room = Room.objects.get(self.current.task_data['room_key'])
        dzps = sorted(DerslikZamanPlani.objects.filter(derslik=self.current.room),
                      key=lambda d: (d.gun, d.baslangic_saat, d.baslangic_dakika))

        zaman_plani = list()

        for dz in dzps:
            durum = dict()
            durum['key'] = dz.key
            durum['saat'] = '{baslangic}-{bitis}'.format(
                baslangic=format_time(time(int(dz.baslangic_saat), int(dz.baslangic_dakika))),
                bitis=format_time(time(int(dz.bitis_saat), int(dz.bitis_dakika))),
            )
            durum['gun'] = dz.gun
            durum['durum'] = dz.durum
            zaman_plani.append(durum)

        item = {'derslik_key': self.current.room.key,
                'name': self.current.room.name,
                'kapasite': self.current.room.capacity,
                'zaman_plani': zaman_plani,
                'derslikler': self.derslik_listele()}

        self.output['derslik_zaman_tablosu'] = item
        self.listele()

    def sinav_derslik_zt(self):
        room = Room.objects.get(self.current.task_data['room_key'])
        dsp = DerslikSinavPlani.objects.filter(derslik=room)
        zaman_plani = list()

        for dz in dsp:
            durum = dict()
            durum['key'] = dz.key
            durum['saat'] = dz.zaman_dilimi.baslama_saat + ':' + dz.zaman_dilimi.baslama_dakika + '-' + \
                            dz.zaman_dilimi.bitis_saat + ':' + dz.zaman_dilimi.bitis_dakika
            durum['gun'] = dz.gun
            durum['durum'] = dz.durum
            zaman_plani.append(durum)

        item = {'derslik_key': room.key,
                'name': room.name,
                'kapasite': room.exam_capacity,
                'zaman_plani': zaman_plani,
                'derslikler': self.sinav_derslik_listele()}

        self.output['derslik_zaman_tablosu'] = item
        self.listele()


    def sinav_degisiklik_kaydet(self):
        self.save_changes(DerslikSinavPlani, self.current.input['change'])
        kayit_durum = True
        self.current.output['kayit_durum'] = kayit_durum

    def zaman_degisiklik_kaydet(self):
        self.save_changes(ZamanCetveli, self.current.input['change'])
        kayit_durum = True
        self.current.output['kayit_durum'] = kayit_durum

    def derslik_degisiklik_kaydet(self):
        self.save_changes(DerslikZamanPlani, self.current.input['change'])
        kayit_durum = True
        self.current.output['kayit_durum'] = kayit_durum

    def save_changes(self, model, change):
        """"
        save timetable state when user switches room's availability  options

        .. code-block:: python
            # request:
            {
                'change':{
                    'key': string   # key,
                    'durum': int    # 1 = Herkese Acik,
                    'cmd': 'kaydet'}
            }

             # response:
            {
                'kayit_durum': Boolean
            }

        """
        obj = model.objects.get(change['key'])
        obj.durum = change['durum']
        obj.save()

    def onaya_gonder(self):
        _form = JsonForm(title=_(u'Programı Bölüm Başkanına yollamak istiyor musunuz?'))
        _form.evet = fields.Button(_(u'Evet'), cmd='evet')
        _form.hayir = fields.Button(_(u'Hayır'), cmd='hayir')
        self.form_out(_form)

    def mesaj(self):
        msg = {"title": _(u'Onay İçin Gönderildi!'),
               "body": _(u'Talebiniz Başarıyla iletildi.')}
        self.current.task_data['LANE_CHANGE_MSG'] = msg

    def ogretim_gorevlisi_kontrol_et(self):
        """
        Switch lecturer or select default one

        .. code-block:: python
            # request:
            {
                'secili_og_elemani':{
                    'key': string   #  personel key,
                    'cmd': string}
            }

        """
        # default select first person
        if 'cmd' in self.current.task_data and self.current.task_data['cmd'] == 'kontrol':
            self.current.task_data['ogretim_elemani_key'] = self.current.input['secili_og_elemani']['key']
        else:
            self.current.task_data['ogretim_elemani_key'] = \
                (OgElemaniZamanPlani.objects.filter(birim=self.current.role.unit)[0]).key

    def derslik_kontrol_et(self):
        self.derslik_sec_kontrol('kontrol', DerslikZamanPlani)

    def sinav_derslik_kontrol_et(self):
        self.derslik_sec_kontrol('kontrol', DerslikSinavPlani)

    def ogretim_elemani_onay_ekrani(self):
        if 'red_aciklamasi' in self.current.task_data:
            del(self.current.task_data['red_aciklamasi'])
        self.ogretim_elemani_zt()
        self.output['ogretim_elemani_zt']['readonly'] = True
        self.form_out(OnayForm())

    def derslik_onay_ekrani(self):
        """
        read only control and approve screen

        .. code-block:: python
            # response:
            {
                'derslik_zaman_tablosu': {
                    'derslik_key': string   # derslik key
                    'name': string,     # room name,
                    'kapasite': int,   # capacity,
                    'zaman_plani': [{
                        'key': string,     # derslik zaman plani key
                        'saat': string,  # 10:00-12:00,
                        'gun': int,     # 1 = pazartesi,
                        'durum': int    # 2 = Bolume Ait,
                        }],
                    'derslikler': [{
                        'name': string,
                        'key': string}]}
            }
        """

        if 'red_aciklamasi' in self.current.task_data:
            del(self.current.task_data['red_aciklamasi'])
        self.derslik_zt()
        self.output['derslik_zaman_tablosu']['readonly'] = True
        self.form_out(OnayForm())

    def sinav_derslik_onay_ekrani(self):
        if 'red_aciklamasi' in self.current.task_data:
            del(self.current.task_data['red_aciklamasi'])
        self.sinav_derslik_zt()
        self.output['derslik_zaman_tablosu']['readonly'] = True
        self.form_out(OnayForm())

    def red_aciklama_yaz(self):
        _form = JsonForm()
        _form.mesaj = fields.String(_(u'Açıklama'))
        _form.gonder = fields.Button(_(u'Gönder'))
        self.form_out(_form)

    def red_aciklamasini_gonder(self):
        self.current.task_data['red_aciklamasi'] = self.current.input['form']['mesaj']
        msg = {"title": _(u'Red Açıklaması Gönderildi!'),
               "body": _(u'İşleminiz başarıyla gerçeklesmistir')}
        self.current.task_data['LANE_CHANGE_MSG'] = msg

    def bilgilendirme(self):
        msg = {"type": "info",
               "title": _(u'Talebiniz Onaylandı!'),
               "msg": _(u'Gönderdiğiniz zaman tablosu Bölüm Başkanı tarafından onaylandı.')}

        self.current.output['msgbox'] = msg

    def onayla(self):
        msg = {"title": _(u'Zaman Tablosunu Onayladınız!'),
               "body": _(u'Bölüm ders programı koordinatörüne onaylama iletildi.')}

        self.current.task_data['LANE_CHANGE_MSG'] = msg


# -*- coding: utf-8 -*-
# from zato.server.service import Service
from ulakbus.lib.unitime import ExportCourseTimetable
from ulakbus.models import User
import xml.etree.ElementTree as ET
import subprocess
import os
from ulakbus.lib.common import ders_programi_doldurma
import shutil
import httplib
from ulakbus.services.ulakbus_service import UlakbusService


class DersProgramiStartSolver(UlakbusService):
    """Ders programı planlamasını arka planda başlatır."""

    HAS_CHANNEL = True

    def handle(self):
        solver_service = DersProgramiExecuteSolver().name
        self.invoke_async(solver_service, payload=self.request.payload)
        self.response.status_code = httplib.OK
        self.response.payload = {'status': 'ok', 'result': 'Ders planlaması başlatıldı'}


class DersProgramiExecuteSolver(UlakbusService):
    """Bir bölüm için ders programı hesaplaması yapar.

    Bu servisi çalıştırmak için ders programı hesaplanacak olan bölümün
    yöksis numarası, ve ders planı hesaplamasını başlatan kullanıcının key'i
     verilmelidir. Örneğin:

    >>> requests.post('http://example.com/dersler', json={'bolum': 124150, 'kullanici': 'Lpdk2348cTfWeTTXY22asd', 'url': ''})

    Servis, ders programını hesaplayarak sonucunu kaydedecektir. İşlem
    bittiğinde, servis durumunu bildiren bir sonuç verir. Örneğin başarılı
    olması durumunda gelecek sonuç:

        {'status': 'ok', 'result': 'Tüm dersler yerleştirildi'}

    Başarısız bir sonuç örneği:

        {'status': 'fail', 'result': 'Solver çalıştırılırken hata oluştu'}

    Eğer servis HTTP üzerinden kullanılıyorsa HTTP durum kodlarına da bakılabilir.

    Servis tamamlandıktan sonra, işlemi başlatan kullanıcıyı notification göndererek uyaracaktır.
    Kullanıcıya gönderilecek olan mesajı değiştirmek için, servise gönderilen istekte 'baslik' ve 'mesaj'
    alanları kullanılabilir.
    """

    HAS_CHANNEL = False

    _SOLVER_DIR = '/opt/zato/solver'

    def handle(self):
        bolum_yoksis_no = int(self.request.payload['bolum'])
        kullanici_key = self.request.payload['kullanici']
        url = self.request.payload['url']
        export_dir = os.path.join(self._SOLVER_DIR, str(bolum_yoksis_no))
        try:
            status, result = self._handle(bolum_yoksis_no, kullanici_key, url, export_dir)
        except Exception as e:
            shutil.rmtree(export_dir)
            raise e

        # İşlem sonucunu hem HTTP durumu olarak, hem de yanıtın içine yaz
        self.response.status_code = status
        status_msg = 'ok' if status == httplib.OK else 'fail'
        self.response.payload = {'status': status_msg, 'result': result}

    def _handle(self, bolum_yoksis_no, kullanici_key, url, export_dir):
        if not os.path.isdir(self._SOLVER_DIR):
            os.mkdir(self._SOLVER_DIR)

        if os.path.isdir(export_dir):
            return httplib.CONFLICT, '%i yöksis no\'lu bölüm için çalışan bir solver var' % bolum_yoksis_no

        # Derslerin export'unu al
        data_set = ExportCourseTimetable(bolum=bolum_yoksis_no)
        os.mkdir(export_dir)
        data_set.EXPORT_DIR = export_dir
        data_set.run()

        # export edilen XML dosyasini, solverda calistirir.
        export_file = os.path.join(export_dir, '%i.xml' % bolum_yoksis_no)
        os.chdir(self._SOLVER_DIR)

        p = subprocess.Popen(
            ["java", "-Xmx1g", "-jar", "cpsolver-1.3.79.jar", "great-deluge.cfg", export_file, export_dir],
            stdout=subprocess.PIPE, universal_newlines=True)

        os.chdir(export_dir)
        output_folder = ''
        p.wait()
        # Solver'ın çıktısını inceleyerek olası hata mesajlarını yakala,
        # solver'ın çözümü yazdığı yeri bul
        for line in p.stdout:
            if 'test failed' in line.lower():
                return httplib.INTERNAL_SERVER_ERROR, 'XML exportları hatalı'
            elif line.startswith("Output folder:"):
                output_folder = line.split(":")[1][1:].strip()

        if output_folder == '':
            return httplib.INTERNAL_SERVER_ERROR, 'Solver çalıştırılırken hata oluştu'

        # Sonuçları oku
        root = ET.parse(os.path.join(output_folder, 'solution.xml')).getroot()
        ders_programi_doldurma(root)
        shutil.rmtree(export_dir)

        # Çözümü isteyen kullanıcıyı bilgilendir
        kullanici = User.objects.get(key=kullanici_key)
        baslik = self.request.payload.get('baslik', 'Ders planlaması tamamlandı')
        mesaj = self.request.payload.get('mesaj', 'Ders planlaması işlemi tamamlandı.')
        kullanici.send_notification(baslik, mesaj, url=url)
        return httplib.OK, mesaj



# -*-  coding: utf-8 -*-
# Copyright (C) 2015 ZetaOps Inc.
#
# This file is licensed under the GNU General Public License v3
# (GPLv3).  See LICENSE.txt for details.

import datetime
from math import floor
from ..models import AkademikTakvim, ObjectDoesNotExist, Unit, Room, DersEtkinligi, SinavEtkinligi, User
from zengine.lib.translation import gettext as _
from zengine.lib.cache import Cache
import random
import hashlib
import re

# Dakika cinsinden her bir slotun uzunluğu. Ders planlamada kullanılan en küçük zaman birimi.
SLOT_SURESI = 5

# CPSolver'ın ID alanlarında kabul ettiği maximum değer
SOLVER_MAX_ID = 900000000000000000

# En az bir büyük harf
# En az bir küçük harf
# En az bir sayı
# En az bir özel karakter
# En az 8 en fazla 100 karakter
# Türkçe karakter içermemesi

parola_kalibi = re.compile(
    "^(?=.*?\d)(?=.*?[A-Z])(?=.*?[a-z])(?=.*?[\(\)\[\]\{\}!@#\$%\^&\*\+=\-§±_~\/|\\"
    "><\.,:;≤≥])[A-Za-z\d\(\)\[\]\{\}!@#\$%\^&\*\+=\-§±_~\/|\\><\.,:;≤≥]{8,100}$")

e_posta_kalibi = re.compile('[^@]+@[^@]+\.[^@]+')


def saat2slot(saat):
    return saat * 60 / SLOT_SURESI


def timedelta2slot(td):
    """timedelta cinsinden süreyi slot cinsine dönüştürür.

    Slot, CPSolver tarafından işlenen en küçük zaman birimidir.

    Args:
        td (datetime.timedelta):

    Returns:
        int: Slot cinsinden süre.
    """
    dakika = td.seconds / 60
    return dakika / SLOT_SURESI


def datetime2timestamp(dt):
    """Bir datetime objesini, saat dilimi bilgisi olmayan bir POSIX timestamp'ine dönüştürür.

    Bu fonksiyonun verdiği sonuç, datetime.datetime.utcfromtimestamp
    methodu ile geri okunabilir. Yani,
    >>> dt == datetime.datetime.utcfromtimestamp(datetime2timestamp(dt))
    True

    Bu fonksiyon saat dilimlerini dikkate almadan çalışmaktadır, bu nedenle bu fonksiyonun
    sonuçlarının farklı saat dilimleri arasında kullanılması sorun çıkaracaktır. Aynı nedenle,
    bu fonksiyonun verdiği timestampler POSIX ile uyumlu değildir.

    Args:
        dt (datetime.datetime): Dönüştürülecek datetime objesi

    Returns:
        float: Karşılık gelen POSIX timestamp'i
    """
    return (dt - datetime.datetime(1970, 1, 1)).total_seconds()


def get_akademik_takvim(unit, ogretim_yili):
    # verilen ogretim yilina gore guncel akademik
    # dondurmesi icin ogretim_yili parametresi eklenmistir.
    try:
        akademik_takvim = AkademikTakvim.objects.get(birim_id=unit.key, ogretim_yili=ogretim_yili)
        return akademik_takvim
    except ObjectDoesNotExist:
        yoksis_key = unit.parent_unit_no
        birim = Unit.objects.get(yoksis_no=yoksis_key)
        return get_akademik_takvim(birim, ogretim_yili)


def ders_programi_doldurma(root):
    # inst = [child for child in root.iter('instructor') if child.get('solution') == 'true']

    # room = [child for child in root.iter('room') if child.get('solution') == 'true']

    # time = [child for child in root.iter('time') if child.get('solution') == 'true']

    cls = [child for child in root.iter('class') for i in child.iter('instructor') if
           i.get('solution') == 'true']

    for child in cls:
        ders_etkinlik = DersEtkinligi.objects.get(unitime_key=child.get('id'))
        ders_etkinlik.solved = True

        for room in child.iter('room'):
            if room.get('solution') == 'true':
                room = Room.objects.get(unitime_key=room.get('id'))
                ders_etkinlik.room = room
                break

        for time in child.iter('time'):
            if time.get('solution') == 'true':
                day = time.get('days')
                for gun, k in enumerate(day):
                    if k == '1':
                        ders_etkinlik.gun = gun + 1

                start = int(time.get('start'))
                length = int(time.get('length'))
                duration = (start * SLOT_SURESI) / 60

                saat = "%02d" % floor(duration)
                ders_etkinlik.baslangic_saat = str(saat)
                dakika = "%02d" % (60 * (duration % 1))
                ders_etkinlik.baslangic_dakika = dakika

                duration = start + length
                duration = (duration * SLOT_SURESI) / 60
                saat = "%02d" % floor(duration)
                ders_etkinlik.bitis_saat = str(saat)
                dakika = "%02d" % (60 * (duration % 1))
                ders_etkinlik.bitis_dakika = dakika

        ders_etkinlik.save()


def sinav_etkinlikleri_oku(root):
    """CPSolver tarafından çözülen bir sınav planını okur.

    Args:
        root (xml.etree.ElementTree.Element): Çözülmüş
            sınav planının root elemanı.
    """
    periods = root.find('periods')
    zamanlar = {}
    for period in periods.iter('period'):
        baslangic_s, bitis_s = period.get('time').split(' ')
        baslangic = datetime.datetime.utcfromtimestamp(float(baslangic_s))
        id_ = period.get('id')
        zamanlar[id_] = baslangic

    exams = root.find('exams')
    for exam in exams.iter('exam'):
        assignment = exam.find('assignment')
        if assignment is not None:
            etkinlik = SinavEtkinligi.objects.get(unitime_key=exam.get('id'))
            period_id = assignment.find('period').get('id')
            etkinlik.tarih = zamanlar[period_id]
            etkinlik.solved = True
            for period in assignment.iter('room'):
                room = Room.objects.get(unitime_key=period.get('id'))
                etkinlik.SinavYerleri.add(room=room)
            etkinlik.save()


def aktivasyon_kodu_uret():
    """
    İki tane 12 karakterli random sayı ile anlık zaman bilgisinin birleşiminden
    rastgele bir string oluşturulur. Bu string sha1 ile hashlenir ve 40
    karakterli bir aktivasyon kodu üretilir.

    """
    rastgele_sayi = "%s%s%s" % (str(random.randrange(100000000000)),
                                str(datetime.datetime.now()),
                                str(random.randrange(100000000000)))

    hash_objesi = hashlib.sha1(rastgele_sayi)
    aktivasyon_kodu = hash_objesi.hexdigest()

    return aktivasyon_kodu


def parola_uygunlugu(parola):
    """
    Belirlenen parola kalıbına göre verilen parolanın uyup uymadığı test edilir.
    Parolanın en az bir sayı, bir büyük harf, bir küçük harf, bir özel karakter içerdiği ve
    Türkçe karakter içermediği test edilir.

    Args:
        parola: Yeni parola

    Returns:
        (bool): True ya da False

    """
    return bool(re.match(parola_kalibi, parola))

def e_posta_uygunlugu(e_posta):
    """
    Belirlenen e_posta adresi kalıbına göre verilen parolanın uyup uymadığı test edilir.

    Args:
        e_posta: Test edilecek e_posta adresi

    Returns:
        (bool): True ya da False

    """
    return bool(re.match(e_posta_kalibi, e_posta))



def kullanici_adi_var_mi(yeni_k_adi):
    """
    Verilen kullanıcı adının başka bir kullanıcı
    tarafından kullanıp kullanılmadığını kontrol eden method.
    Eğer kullanılıyorsa True, kullanılmıyorsa False döndürülür.

    Args:
        yeni_kullanici_adi: Yeni kullanıcı adı

    Returns:
        (bool) True ya da False

    """
    try:
        User.objects.get(username=yeni_k_adi)
        return True
    except ObjectDoesNotExist:
        return False


def kullanici_adi_kontrolleri(eski_k_adi, yeni_k_adi, guncel_k_adi):
    """
    Kullanıcı adı uygunluk kontrolleri.

    Args:
        guncel_k_adi(str): current nesnesi altindaki güncel kullanıcı adı
        eski_k_adi(str): formdan gelen kullanıcı adı.
        yeni_k_adi(str): formdan gelen yeni belirlenecek olan kullanıcı adı.

    Returns:
        (bool) True ya da False
        msg(str): hata mesajı

    """
    if guncel_k_adi != eski_k_adi:
        return False, _(u'Kullanıcı adınızı yanlış girdiniz. Lütfen tekrar deneyiniz.')

    if eski_k_adi == yeni_k_adi:
        return False, _(u'Yeni kullanıcı adınız ile eski kullanıcı adınız aynı olmamalıdır.')

    if kullanici_adi_var_mi(yeni_k_adi):
        return False, _(u"""Böyle bir kullanıcı adı bulunmaktadır.
        Lütfen başka bir kullanıcı adı deneyiniz.""")

    return True, None


def parola_kontrolleri(yeni_parola, yeni_parola_tekrar, kullanici=None, eski_parola=None):
    """
    Parola uygunluk kontrolleri.

    Args:
        yeni_parola(str): formdan gelen yeni belirlenecek olan parola
        yeni_parola_tekrar(str): formdan gelen yeni parolanın tekrarı
        kullanici: User objesi
        eski_parola(str): formdan gelen kullanının güncel olarak girdigi parolası

    Returns:
        (bool) True ya da False
        msg(str): hata mesajı

    """

    if eski_parola and not kullanici.check_password(eski_parola):
        return False, _(u'Kullanmakta olduğunuz parolanızı yanlış girdiniz.')
    if yeni_parola != yeni_parola_tekrar:
        return False, _(u'Yeni parolanız ve tekrarı uyuşmamaktadır.')
    if eski_parola and eski_parola == yeni_parola:
        return False, _(u'Yeni parolanız ile eski parolanız aynı olmamalıdır.')
    if not parola_uygunlugu(yeni_parola):
        return False, _(u"""Girmiş olduğunuz parola kurallara uymamaktadır.
        Lütfen parola kural setini dikkate alarak tekrar deneyiniz.""")

    return True, None


class ParolaSifirlama(Cache):
    """
    Parola sıfırlamak için kullanılan cache objesi.

    """
    PREFIX = 'PARE'
    SERIALIZE = False


class EPostaDogrulama(Cache):
    """
    E-Posta doğrulamak için kullanılan cache objesi.

    """
    PREFIX = 'EMVR'
    SERIALIZE = False



# -*-  coding: utf-8 -*-
"""
"""

# Copyright (C) 2015 ZetaOps Inc.
#
# This file is licensed under the GNU General Public License v3
# (GPLv3).  See LICENSE.txt for details.

from datetime import datetime
from ulakbus.lib.cache import GuncelDonem


def prepare_choices_for_model(model, **kwargs):
    """Model için Seçenekler Hazırla

    Args:
        model: Model
        **kwargs: Keyword argümanları

    Returns:
        Keyword argümanlara göre filtrelenmiş modelin,
        key ve __unicode__ method değerlerini

    """

    return [(m.key, m.__unicode__()) for m in model.objects.filter(**kwargs)]


def convert_model_object_titlemap_item(m):
    """
    Callable nesneler ile titlemap uretirken, callable method geriye name, value
    keylerine sahip bir dict dondurmeli. Bu method verilen nesnenin key ve unicode
    methodlarindan bir dict dondurur.

    Args:
        m (Model): model instance
    Returns:
        (dict) name value keyleri ve karsilik gelen degerlerden olusan dict

    """
    return {"name": m.__unicode__(), "value": m.key}


class WFValues(object):
    """
    WF başlangıcında, tüm iş akışı boyunca kullanılacak dataların
    set edilmesini sağlayan methodları içeren class.
    """

    @staticmethod
    def assign_wf_initial_values(current):
        """
        WF çalıştırılırken wf current'ının task_data'sına başlangıçta istenilen
        dataların konulmasını ve tüm adımlarda bu dataların kullanılmasını sağlar.

        Args:
            current (Current): current nesnesi
        """

        current.task_data['wf_initial_values'] = {
            'guncel_donem': GuncelDonem().get_or_set(),
            'started': True,
            'start_date': datetime.now().strftime("%Y-%m-%d %H:%M:%S")
        }



# -*-  coding: utf-8 -*-
"""
"""

# Copyright (C) 2015 ZetaOps Inc.
#
# This file is licensed under the GNU General Public License v3
# (GPLv3).  See LICENSE.txt for details.
from datetime import datetime
from ulakbus.lib.view_helpers import prepare_choices_for_model
from zengine.views.crud import CrudView
from zengine.forms import JsonForm
from zengine.forms.fields import Integer, Button
from zengine.models import BPMNWorkflow, Task
from zengine.lib.translation import gettext as _, gettext_lazy as __
from ulakbus.models import AbstractRole, Role, Unit


MESAJ = {
    'type': 'info', "title": _(u'İşlem Başarılı'),
    "msg": _(u'İlgili birime bağlı kişilere iş akışı aktarıldı.')
}


def format_date(start_date, finish_date):
    """

    Args:
        start_date: datetime (form input)
        finish_date: datetime (form input)

    Returns:
        start: string (02.10.2010)
        finish: string (10.10.2010)

    """
    start = datetime.strptime(start_date, '%d.%m.%Y')
    finish = datetime.strptime(finish_date, '%d.%m.%Y')
    return start, finish


class FormTask(JsonForm):
    class Meta:
        include = ['abstract_role', 'start_date', 'finish_date']

    gonder = Button(__(u"Onayla"))
    geri = Button(__(u"Geri"), cmd='geri')


form_include_A = ['unit', 'object_query_code', 'object_type', 'recursive_units']
form_include_B = ['get_roles_from', 'unit', 'object_query_code', 'object_type', 'recursive_units']
form_include_C = ['get_roles_from']
form_include_D = ['get_roles_from', 'unit', 'recursive_units']

help_text_A = __(
u"""Modelin seçildiği, role özel atanan iş akışı atama formu:
* Model tipi seçilmesi zorunludur.
* Soyut rol secilmesi zorunludur.
* Arama sorgusu yazılması zorunludur ve "role" ile başlamalıdır.
Örnek: Şube modelini kullanan bir iş akışının okutmanlara özel gitmesi için sorgunun;
okutman=role.user.personel.okutman şeklinde olmalıdır.
* Birim seçilmesi zorunludur. Eğer seçilen birim Fakülte ise
alt birimlerde ki rolleri getir seçeneği işaretlenmesi zorunludur.
* İş akışının başlangıç ve bitiş zamanlari girilmesi zorunludur.
""")

help_text_B = __(
u"""Modelin seçildiği, aynı soyut role sahip kişilere aynı iş akışı atama formu:
* Model tipi seçilmesi zorunludur.
* Arama sorgusu yapılması zorunlu değildir.
Eğer bir sorgu yapılacak ise sorgu kodu "role" ile başlamamalıdır.
Örnek: personel modelini kullanan bir iş akışı için sorgu şu şekilde olabilir; personel_turu=2
* Eğer rolleri getir secenegi kullanılırsa, birim ile soyut rol kullanılmaz.
Bu tam tersi durum içinde geçerlidir.
* İş akışının başlangıç ve bitiş zamanlari girilmesi zorunludur.

""")
help_text_C = __(
u"""Modelin olmadığı, aynı soyut role sahip kişilere aynı iş akışı atama formu:
* Burada iki seçenek vardır. Ya rolleri getir secenegi kullanılır ya da soyut rol seçeneği.
""")
help_text_D = __(
u"""Modelin olmadığı aynı soyut role sahip, role özel atanan iş akışı atama formu:
 * Birim secilirse, soyut rol seceneğininde secilmezi zorunludur.
 Eğer birim fakülte ise alt birimlerde ki rolleri getir seçeneğinin de seçilmesi zorunludur.
 get_roles_from seçenegi boş bırakılır.
 * rolleri getir seçeneği seçilirse, diger seçenekler boş bırakılır
""")


class WorkflowManagement(CrudView):
    class Meta:
        model = 'Task'

    def wf_sec(self):
        _secim_wf = prepare_choices_for_model(BPMNWorkflow, programmable=True)
        _form = JsonForm(title=_(u"İş Yöneticisi"))
        _form.workflow = Integer(title=_(u"Workflow Seçiniz"), choices=_secim_wf)
        _form.gonder = Button(_(u"İlerle"))

        self.form_out(_form)

    def wf_zamanla(self):
        self.current.task_data['wf_name'] = self.input['form']['workflow']
        workflow = BPMNWorkflow.objects.get(name=self.current.task_data['wf_name'])
        include_fields = 'form_include_%s' % workflow.task_type
        help_text = 'help_text_%s' % workflow.task_type
        _form = FormTask(self.object, current=self.current)
        _form.Meta.include.extend(globals()[include_fields])
        _form.title = workflow.title
        _form.help_text = globals()[help_text]
        self.form_out(_form)

    def bilgi_ekrani(self):
        wf = BPMNWorkflow.objects.get(name=self.current.task_data['wf_name'])

        self.set_form_data_to_object()
        self.object.wf = wf
        self.object.name = wf.title or wf.name
        self.object.run = True
        self.save_object()

        self.current.output['msgbox'] = {"type": _(u"info"),
                                         "title": _(u"İşlem Başarılı"),
                                         "msg": _(u"İş akışı ilgili rollere aktarıldı.")}



# -*-  coding: utf-8 -*-

# Copyright (C) 2015 ZetaOps Inc.
#
# This file is licensed under the GNU General Public License v3
# (GPLv3).  See LICENSE.txt for details.

"""HITAP Tazminat Guncelle

Hitap'a personelin Tazminat bilgilerinin guncellemesini yapar.

"""

from ulakbus.services.personel.hitap.hitap_guncelle import HITAPGuncelle


class HizmetTazminatGuncelle(HITAPGuncelle):
    """
    HITAP Guncelleme servisinden kalıtılmış Hizmet Tazminat Bilgi Guncelleme servisi

    """
    HAS_CHANNEL = True

    def handle(self):
        """Servis çağrıldığında tetiklenen metod.

        Attributes:
            service_name (str): İlgili Hitap sorgu servisinin adı
            service_dict (dict): Request yoluyla gelen kayıtlar,
                    HizmetTazminatUpdate servisinin alanlarıyla eşlenmektedir.
                    Filtreden geçecek tarih alanları listede tutulmaktadır.
        """

        self.service_name = 'HizmetTazminatUpdate'
        self.service_dict = {
            'fields': {
                'kayitNo': self.request.payload.get('kayit_no', ''),
                'gorev': self.request.payload.get('gorev', ''),
                'kadrosuzluk': self.request.payload.get('kadrosuzluk', ''),
                'makam': self.request.payload.get('makam', ''),
                'tckn': self.request.payload.get('tckn', ''),
                'temsil': self.request.payload.get('temsil', ''),
                'unvanKod': self.request.payload.get('unvan_kod', ''),
                'tazminatTarihi': self.request.payload.get('tazminat_tarihi', ''),
                'tazminatBitisTarihi': self.request.payload.get('tazminat_bitis_tarihi', ''),
                'kurumOnayTarihi': self.request.payload.get('kurum_onay_tarihi', '')
            },
            'date_filter': ['tazminatTarihi', 'tazminatBitisTarihi', 'kurumOnayTarihi'],
            'required_fields': ['kayitNo', 'tckn', 'unvanKod', 'tazminatTarihi', 'kurumOnayTarihi']
        }
        super(HizmetTazminatGuncelle, self).handle()



# -*-  coding: utf-8 -*-
"""
"""

# Copyright (C) 2015 ZetaOps Inc.
#
# This file is licensed under the GNU General Public License v3
# (GPLv3).  See LICENSE.txt for details.
from ulakbus.models import OgrenciProgram, OgrenciDersi, Sinav, DegerlendirmeNot, Ogrenci, Role
from ulakbus.views.ders.ders import prepare_choices_for_model
from zengine import forms
from zengine.forms import fields
from zengine.views.crud import CrudView
from zengine.lib.translation import gettext as _, gettext_lazy
from ulakbus.lib.role import AbsRole
from ulakbus.views.ogrenci.ogrenci import ABSTRACT_ROLE_LIST


class NotDuzenlemeForm(forms.JsonForm):
    """
    `NotDuzenleme`` sınıfı için form olarak kullanılacaktır. Form,
    include listesinde, aşağıda tanımlı alanlara sahiptir.

    """

    class Meta:
        include = ['puan']

    kaydet = fields.Button(gettext_lazy(u'Kaydet'))


class NotDuzenleme(CrudView):
    """Öğrenci İşleri Not Düzenleme İş Akışı

   Öğrenci İşleri Not Düzenleme  iş akışı 6 adımdan oluşmaktadır.

   * Fakülte Yönetim Karar No
   * Öğrenci Program Seç
   * Ders Seç
   * Sınav Seç
   * Not Düzenle
   * Bilgilendir

   Bu iş akışında kullanılan metotlar şu şekildedir.

   Fakülte Yönetim Karar No:
     Fakülte Yönetim Kurulu tarafından belirlenen karar no girilir.

   Program Seç:
      Öğrencinin kayıtlı olduğu öğrenci program seçilir.

    Ders Seç:
      Öğrencinin kayıtlı olduğu derslerden biri seçilir

    Sınav Seç:
      Derse ait sınavlardan biri seçilir.

    Not Düzenle:
      Seçilen sınavın puanı düzenlenir.

    Bilgilendir:
      Not düzenleme işlemi tamamlandıktan sonra ekrana bilgilendirme mesajı basılır.

    Bu sınıf ``CrudView`` extend edilerek hazırlanmıştır. Temel model ``DeğerlendirmeNot``
    modelidir. Meta.model bu amaçla kullanılmıştır.

    Adımlar arası geçiş manuel yürütülmektedir.

    """

    class Meta:
        model = 'DegerlendirmeNot'

    def fakulte_yonetim_karari(self):
        """
        Fakülte Yönetim Kurulu tarafından belirlenen karar no girilir.

        """

        # TODO: Fakülte yönetim kurulunun kararı loglanacak.
        self.current.task_data['ogrenci_id'] = self.current.input['id']
        _form = forms.JsonForm(current=self.current,
                               title=_(u'Fakülte Yönetim Kurulunun Karar Numarasını Giriniz.'))
        _form.karar = fields.String(_(u'Karar No'), index=True)
        _form.kaydet = fields.Button(_(u'Kaydet'))
        self.form_out(_form)

    def program_sec(self):
        """
        Öğrencinin kayıtlı olduğu öğrenci program seçilir.

        """

        _form = forms.JsonForm(current=self.current, title=_(u'Program Seçiniz.'))
        _choices = prepare_choices_for_model(OgrenciProgram,
                                             ogrenci_id=self.current.task_data['ogrenci_id'])
        _form.program = fields.Integer(choices=_choices)
        _form.onayla = fields.Button('Seç')
        self.form_out(_form)

    def ders_sec(self):
        """
        Öğrencinin kayıtlı olduğu derslerden biri seçilir.

        """

        program_id = self.current.input['form']['program']
        self.current.task_data['program_id'] = self.current.input['form']['program']
        _form = forms.JsonForm(current=self.current, title=_(u'Ders Seçiniz.'))
        _choices = prepare_choices_for_model(OgrenciDersi, ogrenci_program_id=program_id)
        _form.ders = fields.Integer(choices=_choices)
        _form.onayla = fields.Button(_(u'Seç'))
        self.form_out(_form)

    def sinav_sec(self):
        """
        Derse ait sınavlardan biri seçilir.

        """

        self.current.task_data['ders_id'] = self.current.input['form']['ders']
        ogrenci_dersi = OgrenciDersi.objects.get(self.current.task_data['ders_id'])

        _form = forms.JsonForm(current=self.current, title=_(u'Sınav Seçiniz.'))
        _choices = prepare_choices_for_model(Sinav, sube_id=ogrenci_dersi.sube.key)
        _form.sinav = fields.Integer(choices=_choices)
        _form.onayla = fields.Button(_(u'Seç'))
        self.form_out(_form)

    def not_duzenle(self):
        """
        Seçilen sınavın puanı düzenlenir.

        """

        sinav_id = self.current.input['form']['sinav']
        sinav = Sinav.objects.get(sinav_id)
        self.current.task_data['sinav'] = sinav.__unicode__()
        ogrenci_id = self.current.task_data['ogrenci_id']
        degerlendirme_not = \
            DegerlendirmeNot.objects.get(sinav_id=sinav_id, ogrenci_id=ogrenci_id)
        self.current.task_data['onceki_puan'] = degerlendirme_not.puan

        title = _(u'%(ogrenci)s adlı öğrencinin %(sinav)s sınava ait notunu düzenleyiniz.') % {
            'ogrenci': degerlendirme_not.ogrenci,
            'sinav': degerlendirme_not.sinav,
        }
        _form = NotDuzenlemeForm(degerlendirme_not, current=self.current, title=title)

        self.form_out(_form)

    def bilgilendir(self):
        """
        Not düzenleme işlemi tamamlandıktan sonra ekrana bilgilendirme mesajı basılır.

        """

        ogrenci_id = self.current.task_data['ogrenci_id']
        ogrenci = Ogrenci.objects.get(ogrenci_id)
        yeni_puan = self.current.input['form']['puan']
        sinav = self.current.task_data['sinav']
        onceki_puan = self.current.task_data['onceki_puan']

        self.current.output['msgbox'] = {
            'type': 'info', "title": _(u'Not Düzeltme Tamamlandı'),
            "msg": _(u"""%(ogrenci)s adlı öğrencinin, %(sinav)s sınavına ait
            %(onceki_puan)s olan notu, %(yeni_puan)s ile değiştilmiştir.""") % {
                'ogrenci': ogrenci,
                'sinav': sinav,
                'onceki_puan': onceki_puan,
                'yeni_puan': yeni_puan,
            }
        }
        title = _(u"Not Düzenleme")
        message = _(u"""%(sinav)s sınavınıza ait %(onceki_puan)s olan notu,
                        %(yeni_puan)s ile değiştirilmiştir.""") % {
            'sinav': sinav,
            'onceki_puan': onceki_puan,
            'yeni_puan': yeni_puan
        }
        ogrenci_program = OgrenciProgram.objects.get(self.current.task_data['program_id'])
        birim = ogrenci_program.program.birim
        for role in Role.objects.filter(user=ogrenci.user, unit=birim):
            if role.abstract_role.name in ABSTRACT_ROLE_LIST:
                role.send_notification(message=message, title=title, sender=self.current.user)




# -*-  coding: utf-8 -*-

# Copyright (C) 2015 ZetaOps Inc.
#
# This file is licensed under the GNU General Public License v3
# (GPLv3).  See LICENSE.txt for details.

"""Banka Borç Ödeme

Banka tarafından öğrencinin borçlarını ödemek için kullanılan Zato Servisi.

Yetkilendirme için banka bilgileri,
öğrenci borç ödemesinin gerçekleştirimi için öğrenci numarası ve ödeme bilgileri verilir.

Elde edilen bilgilerle öğrencinin borç ödeme bilgisi sisteme kaydedilir.

Example:

    Servise JSON nesnesi kullanılarak istek gönderilmesi:

    .. code-block:: json

        $ curl http://localhost:11223/banka-borc-getir -d '{
            "banka_kodu": "kod",
            "bank_username": "user",
            "bank_password": "pass",
            "ogrenci_no":"1234567890",
            "sube_kodu":"sube",
            "kanal_kodu":"kanal",
            "mesaj_no":"mesaj",
            "ucret_turu":1,
            "tahakkuk_referans_no":"tahakkuk",
            "tahsilat_referans_no":"tahsilat",
            "odeme_timestamp":"07122015123456",
            "odeme_tutari":214.0
            }'


    İsteğe dönen cevap:

    .. code-block:: json

        $ {"odeme_response": [{
            "tahakkuk_referans_no": "tahakkuk",
            "odeme_timestamp": "07122015123456",
            "odeme_tutari": "214.0",
            "ucret_turu": 1,
            "tahsilat_referans_no": "tahsilat",
            "bank_password": "pass",
            "bank_username": "user",
            "mesaj_no": "mesaj",
            "sube_kodu": "sube",
            "hata_mesaj": null,
            "ogrenci_no": "1234567890",
            "banka_kodu": "kod",
            "kanal_kodu": "kanal",
            "mesaj_statusu": "K"
            }
        ]}


    Servise XML kullanılarak istek gönderilmesi:

    .. code-block:: xml

        $ curl http://localhost:11223/banka-odeme.banka-borc-odeme
            -H "SOAPAction:banka-odeme.banka-borc-odeme"
            -d '<soapenv:Envelope xmlns:soapenv="http://schemas.xmlsoap.org/soap/envelope/"
                xmlns:zato="https://zato.io/ns/20130518">
                    <soapenv:Body>
                        <zato:odeme_request>
                            <zato:banka_kodu>kod</zato:banka_kodu>
                            <zato:bank_username>user</zato:bank_username>
                            <zato:bank_password>pass</zato:bank_password>
                            <zato:ogrenci_no>1234567890</zato:ogrenci_no>
                            <zato:sube_kodu>sube</zato:sube_kodu>
                            <zato:kanal_kodu>kanal</zato:kanal_kodu>
                            <zato:mesaj_no>mesaj</zato:mesaj_no>
                            <zato:ucret_turu>1</zato:ucret_turu>
                            <zato:tahakkuk_referans_no>tahakkuk</zato:tahakkuk_referans_no>
                            <zato:tahsilat_referans_no>tahsilat</zato:tahsilat_referans_no>
                            <zato:odeme_timestamp>07122015123456</zato:odeme_timestamp>
                            <zato:odeme_tutari>214.0</zato:odeme_tutari>
                        </zato:odeme_request>
                    </soapenv:Body>
                </soapenv:Envelope>'


    İsteğe dönen cevap:

    .. code-block:: xml

        $ <?xml version='1.0' encoding='UTF-8'?>
            <soap:Envelope xmlns:soap="http://schemas.xmlsoap.org/soap/envelope/"
            xmlns="https://zato.io/ns/20130518">
            <soap:Body>
                <odeme_response>
                    <zato_env>
                        <cid>K07C4A0D30PHBMBGX18BYAZBZRH7</cid>
                        <result>ZATO_OK</result>
                    </zato_env>
                    <item_list xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance">
                        <item>
                            <mesaj_statusu>K</mesaj_statusu>
                            <hata_mesaj xsi:nil="true"/>
                            <banka_kodu>kod</banka_kodu>
                            <sube_kodu>sube</sube_kodu>
                            <kanal_kodu>kanal</kanal_kodu>
                            <mesaj_no>mesaj</mesaj_no>
                            <bank_username>user</bank_username>
                            <bank_password>pass</bank_password>
                            <ogrenci_no>1234567890</ogrenci_no>
                            <ucret_turu>1</ucret_turu>
                            <tahakkuk_referans_no>tahakkuk</tahakkuk_referans_no>
                            <tahsilat_referans_no>tahsilat</tahsilat_referans_no>
                            <odeme_timestamp>07122015123456</odeme_timestamp>
                            <odeme_tutari>214.0</odeme_tutari>
                        </item>
                    </item_list>
                </odeme_response>
            </soap:Body>
        </soap:Envelope>

"""

from ulakbus.services.common.banka import BankaService
from ulakbus.models.ogrenci import OgrenciProgram, Borc, Odeme
from pyoko.exceptions import ObjectDoesNotExist
import json
from datetime import datetime


class BankaBorcOdeme(BankaService):
    """
    Banka servisinden kalıtılarak gerçekleştirilen Banka Borç Ödeme servisi

    """

    HAS_CHANNEL = True

    def __init__(self):
        super(BankaBorcOdeme, self).__init__()

    class SimpleIO():
        """
        Servis girdilerinin ve çıktılarının belirlendiği yapı.

        Note:
            SimpleIO sınıfında belirlenen özellikler,
            doğrudan yer aldığı servisin özellikleri olmaktadır.

            - odeme_request (str): Servise gelen isteğin (JSON, XML) ismi
            - odeme_response (str): Servisten dönen veriyi içeren (payload)
              cevabın (JSON, XML) ismi
            - mesaj_statusu (str): K (Kabul), R (Ret)
            - hata_mesaji (str): Hata mesajı içeriği veya null
            - banka_kodu (str): Üniversite tarafından bankaya verilen kod
            - bank_username (str): Üniversite tarafından bankaya verilen kullanıcı adı
            - bank_password (str): Üniversite tarafından bankaya verilen şifre
            - sube_kodu (str): Bankaların şubeleri için hali hazırda kullandıkları kodlar
            - kanal_kodu (str): G (Gişe), İ (İnternet), A (ATM), T (AloBanka) vb.
            - mesaj_no (str): Banka tarafından üretilen kod.
            - ogrenci_no (str): Borçları sorgulanan öğrencinin numarası
            - ucret_turu (int): Borcun ne için ödeneceği
            - tahakkuk_referans_no (str): Her tahakkuka verilen referans numarası
            - tahsilat_referans_no (str): Banka tarafından verilen tahsilata ait referans no
            - odeme_timestamp (str): DDMMYYYYHHMMSS formatında ödeme tarihi
            - odeme_tutari (float): Banka tarafından tahsil edilen tutar

        """

        request_elem = 'odeme_request'
        response_elem = 'odeme_response'
        input_required = ('banka_kodu', 'sube_kodu', 'kanal_kodu', 'mesaj_no', 'bank_username',
                          'bank_password', 'ogrenci_no', 'ucret_turu', 'tahakkuk_referans_no',
                          'tahsilat_referans_no', 'odeme_timestamp', 'odeme_tutari')
        output_required = ('banka_kodu', 'sube_kodu', 'kanal_kodu', 'mesaj_no', 'bank_username',
                           'bank_password', 'ogrenci_no', 'ucret_turu', 'tahakkuk_referans_no',
                           'tahsilat_referans_no', 'odeme_timestamp', 'odeme_tutari',
                           'mesaj_statusu', 'hata_mesaj')

    def handle(self):
        """
        Servis çağrıldığında tetiklenen metod.

        Yetkilendirme işleminin sonucuna göre servis çalışmaya devam eder.

        """

        super(BankaBorcOdeme, self).handle()

    def get_data(self):
        """
        Öğrencinin borç ödeme bilgilerini bankadan gelen bilgilere göre sisteme kaydeder.

        Öğrenci numarası ve borcuna ait tahakkuk referans numarasına göre,
        öğrencinin borcuna erişilir. Bankadan gelen ödeme bilgisine göre bu borca ait
        bir ödeme bilgisi oluşturulur.

        Öğrenci sistemde bulunamadıysa veya beklenmeyen bir hatayla karşılaşıldıysa
        cevap olarak hata mesajı döndürülür.

        Raises:
            ObjectDoesNotExist: Öğrenci numarası sistemde kayıtlı değildir.
            Exception: Öğrencinin borç ödeme bilgisi kaydedilirken hatayla karşılaşılmıştır.

        """

        super(BankaBorcOdeme, self).get_data()

        inp = self.request.input
        mesaj_statusu = "R"
        hata_mesaj = None

        try:
            ogrenci_no = inp.ogrenci_no
            ogrenci = OgrenciProgram.objects.get(ogrenci_no=ogrenci_no).ogrenci

            # her borcun referans numarasi olarak 'tahakkuk_referans_no' kullanilir
            borc = Borc.objects.filter(ogrenci=ogrenci,
                                       tahakkuk_referans_no=inp.tahakkuk_referans_no)[0]

            odeme = Odeme()
            odeme.miktar = inp.odeme_tutari
            odeme.para_birimi = 1  # TL
            odeme.aciklama = borc.aciklama
            odeme.odeme_sekli = 3  # Banka
            odeme.odeme_tarihi = datetime.strptime(inp.odeme_timestamp, "%d%m%Y%H%M%S")
            odeme.borc = borc
            odeme.ogrenci = ogrenci
            odeme.banka = self.banka
            odeme.banka_sube_kodu = str(inp.sube_kodu)
            odeme.banka_kanal_kodu = inp.kanal_kodu
            odeme.banka_kanal_kodu = inp.kanal_kodu
            odeme.tahsilat_referans_no = inp.tahsilat_referans_no
            odeme.donem = borc.donem

            odeme.save()
            mesaj_statusu = "K"
            hata_mesaj = None

        except ObjectDoesNotExist:
            self.logger.exception('Ogrenci numarasi bulunamadi.')
            mesaj_statusu = "R"
            hata_mesaj = "Ogrenci numarasi bulunamadi!"

        except Exception:
            self.logger.exception("Odeme kaydedilirken hata olustu.")
            mesaj_statusu = "R"
            hata_mesaj = "Odeme kaydedilirken hata olustu!"

        finally:
            odeme_response = {
                'banka_kodu': inp.banka_kodu,
                'sube_kodu': inp.sube_kodu,
                'kanal_kodu': inp.kanal_kodu,
                'mesaj_no': inp.mesaj_no,
                'bank_username': inp.bank_username,
                'bank_password': inp.bank_password,
                'mesaj_statusu': mesaj_statusu,
                'ogrenci_no': inp.ogrenci_no,
                'ucret_turu': inp.ucret_turu,
                'tahakkuk_referans_no': inp.tahakkuk_referans_no,
                'tahsilat_referans_no': inp.tahsilat_referans_no,
                'odeme_timestamp': inp.odeme_timestamp,
                'odeme_tutari': inp.odeme_tutari,
                'hata_mesaj': hata_mesaj
            }

            self.logger.info("Odeme bilgisi: %s" % json.dumps(odeme_response))
            self.response.payload.append(odeme_response)



# -*- coding: utf-8 -*
from ulakbus.lib.unitime import ExportExamTimetable
from ulakbus.lib.common import sinav_etkinlikleri_oku
from ulakbus.models import User
from xml.etree import ElementTree
import subprocess
import shutil
import os
import httplib
from ulakbus.services.ulakbus_service import UlakbusService


class SinavPlaniStartExamSolver(UlakbusService):
    """Sınav programı planlamasını arka planda başlatır."""

    HAS_CHANNEL = True

    def handle(self):
        solver_service = SinavPlaniExecuteExamSolver().name
        # Bize verilen payload'u Exam Solver servisine aktararak çalıştırıyoruz
        self.invoke_async(solver_service, payload=self.request.payload)
        self.response.status_code = httplib.OK
        self.response.payload = {'status': 'ok', 'result': 'Sınav planlaması başlatıldı'}


class SinavPlaniExecuteExamSolver(UlakbusService):
    """Bir bölüm için sınav planı hesaplaması yapar.

    Bu servisi çalıştırmak için sınav planı hesaplanacak olan bölümün
    yöksis numarası, hesaplamaya dahil edilecek sınavların türlerinin
    listesi, ve sınav planı hesaplamasını başlatan kullanıcının key'i
    verilmelidir. Örneğin:

    >>> requests.post('http://example.com/sinavlar', json={'bolum': 124150, 'sinav_turleri': [1], 'kullanici': 'Lpdk2348cTfWeTTXY22asd', 'url': ''})

    Servis, sınav programını hesaplayarak sonucunu kaydedecektir. İşlem
    bittiğinde, servis durumunu bildiren bir sonuç verir. Örneğin başarılı
    olması durumunda gelecek sonuç:

        {'status': 'ok', 'result': 'Planlama tamamlandı'}

    Başarısız bir sonuç örneği:

        {'status': 'fail', 'result': '00000 yöksis no\'lu bölüm için çalışan bir solver var'}

    Eğer servis HTTP üzerinden kullanılıyorsa HTTP durum kodlarına da bakılabilir.

    Servis tamamlandıktan sonra, işlemi başlatan kullanıcıyı notification göndererek uyaracaktır.
    Kullanıcıya gönderilecek olan mesajı değiştirmek için, servise gönderilen istekte 'baslik' ve 'mesaj'
    alanları kullanılabilir.
    """
    HAS_CHANNEL = False

    _SOLVER_DIR = '/opt/zato/solver'

    def handle(self):
        bolum_yoksis_no = int(self.request.payload['bolum'])
        sinav_turleri = self.request.payload['sinav_turleri']
        kullanici_key = self.request.payload['kullanici']
        url = self.request.payload['url']
        export_dir = os.path.join(self._SOLVER_DIR, '%i' % bolum_yoksis_no)
        
        status, result = self._handle(bolum_yoksis_no, sinav_turleri, kullanici_key, url, export_dir)


        # İşlem sonucunu hem HTTP durumu olarak, hem de yanıtın içine yaz
        self.response.status_code = status
        status_msg = 'ok' if status == httplib.OK else 'fail'
        self.response.payload = {'status': status_msg, 'result': result}

    def _handle(self, bolum_yoksis_no, sinav_turleri, kullanici_key, url, export_dir):
        if not os.path.isdir(self._SOLVER_DIR):
            os.mkdir(self._SOLVER_DIR)

        exporter = ExportExamTimetable(bolum=bolum_yoksis_no,
                                       sinav_turleri=','.join(map(str, sinav_turleri)))

        if os.path.isdir(export_dir):
            return httplib.CONFLICT, '%i yöksis no\'lu bölüm için çalışan bir solver var' % bolum_yoksis_no

        # Sınavların export'unu al
        os.mkdir(export_dir)
        exporter.EXPORT_DIR = export_dir
        exporter.FILE_NAME = '%i.xml' % bolum_yoksis_no
        exporter.run()
        if not os.path.isfile(os.path.join(export_dir, exporter.FILE_NAME)):
            return httplib.INTERNAL_SERVER_ERROR, '%i yöksis no\'lu bölüm için export alınamamış' % bolum_yoksis_no

        # Alınan export'u solver'a çözdür
        os.chdir(self._SOLVER_DIR)
        prefix_file = os.path.join(export_dir, '%i' % bolum_yoksis_no)
        export_file = '%s.xml' % prefix_file
        output_file = '%s.OUTPUT.xml' % prefix_file
        p = subprocess.Popen(
            ['java', '-Xmx1g', '-cp', 'cpsolver-1.3.79.jar', 'org.cpsolver.exam.Test', 'exam-base.cfg',
             export_file, output_file],
        )
        p.wait()

        # Sonuçları oku
        root = ElementTree.parse(output_file).getroot()
        sinav_etkinlikleri_oku(root)
        shutil.rmtree(export_dir)

        # Çözümü isteyen kullanıcıyı bilgilendir
        kullanici = User.objects.get(key=kullanici_key)
        baslik = self.request.payload.get('baslik', 'Sınav planlaması tamamlandı')
        mesaj = self.request.payload.get('mesaj', 'Sınav planlaması işlemi tamamlandı.')
        kullanici.send_notification(baslik, mesaj, url=url)
        return httplib.OK, mesaj



# -*-  coding: utf-8 -*-
"""
"""

# Copyright (C) 2015 ZetaOps Inc.
#
# This file is licensed under the GNU General Public License v3
# (GPLv3).  See LICENSE.txt for details.

from ulakbus.models import *
from ulakbus.views.reports.base import Reporter
from collections import OrderedDict
import datetime
from zengine.lib.translation import gettext_lazy


class PersonelByGender(Reporter):
    TITLE = gettext_lazy(u'Cinsiyete göre personel sayıları')

    def get_objects(self):
        genders = self.convert_choices(Personel().get_choices_for('cinsiyet'))
        result = []
        for val, num in Personel.objects.distinct_values_of('cinsiyet').items():
            try:
                val = int(val)
            except:
                pass
            result.append((genders.get(val, val), num))
        return result


class PersonelByAkademikIdari(Reporter):
    TITLE = gettext_lazy(u'Akademik / İdari Personel Sayısı')

    def get_objects(self):
        choices = self.convert_choices(Personel().get_choices_for('personel_turu'))
        result = []
        for val, num in Personel.objects.distinct_values_of('personel_turu').items():
            try:
                val = int(val)
            except:
                pass
            result.append((choices.get(val, val), num))
        return result


class Kadrolar(Reporter):
    TITLE = gettext_lazy(u'Genel Kadro Durumları')

    def get_objects(self):
        choices = self.convert_choices(Kadro().get_choices_for('durum'))
        result = []
        for val, num in Kadro.objects.distinct_values_of('durum').items():
            try:
                val = int(val)
            except:
                pass
            result.append((choices.get(val, val), num))
        return result


class TerfisiTikananPersonel(Reporter):
    TITLE = gettext_lazy(u"Terfisi Tıkanan Personel Listesi")

    def get_objects(self):
        """
        Terfisi Tikanan Personel icin kriterlerimiz:
            - gorev_ayligi_derecesi, kadro_derecesine esit olmali
            - gorev_ayligi_kademesi 3 ten buyuk olmalidir.

        Metodun amacı terfisi duran personelleri listelemektir. Burada
        görev aylığı kademesi 3 den yukarı olan personel listelenir.
        Sonrada bu personeller içerisinden kadro derecesi görev aylığı
        derecesine eşit olanlar seçilir. Örneğin bir personelin
        kadro derecesi ve görev aylığı derecesi 4 ise o personel 4/3 e
        kadar yükselebilir. 3 e düşemez. 4/4, 4/5 ... şeklinde gider.
        Bu yüzden personeller veritabanından çekilirken görev aylığı kademesi
        3 den yukarı olanlara bakıyoruz.

        Returns:
            personel_list (list): terfisi duran personel listesi

        """

        personel_list = []
        # todo: pyoko bir metod sagladiginda, raw yazilan bu sorguyu duzeltecegiz.
        p_query = Personel.objects.set_params(
            fq="{!frange l=0 u=0 incu=true}sub(gorev_ayligi_derece,kadro_derece)").filter(
            gorev_ayligi_kademe__gte=4)

        for p in p_query:
            personel_record = OrderedDict({})
            personel_record["TCK No"] = p.tckn
            personel_record["Ad"] = "%s %s" % (p.ad, p.soyad)
            personel_record["Personel Tür"] = str(p.personel_turu)
            personel_record["Kadro Derece"] = str(p.kadro.derece if p.kadro.derece else 0)

            personel_record["Görev Aylığı"] = "%i/%i" % (
                p.gorev_ayligi_derece, p.gorev_ayligi_kademe)

            personel_record["Kazanılmış Hak"] = "%i/%i" % (
                p.kazanilmis_hak_derece, p.kazanilmis_hak_kademe)

            personel_record["Emekli Müktesebat"] = "%i/%i" % (
                p.emekli_muktesebat_derece, p.emekli_muktesebat_kademe)

            personel_list.append(personel_record)

        return personel_list


class GorevSuresiBitenPersonel(Reporter):
    TITLE = gettext_lazy(u"Görev Süresi Dolan Personel Listesi")

    def get_objects(self):
        """ 
            Görev süresinin bitme durumu sadece akademik personel için geçerli
            bir durumdur. Personel modelindeki gorev_suresi_bitis alanındaki değerin
            bugünün datetime değerinden küçük olma durumuna bakılır.
        """
        simdi = datetime.date.today()
        bitis_tarihi = simdi + datetime.timedelta(days=120)

        # todo: add order_by
        personeller = Personel.objects.filter(
            gorev_suresi_bitis__lte=bitis_tarihi,
            personel_turu=1
        )
        personel_list = []
        for p in personeller:
            personel = OrderedDict({})
            personel["T.C. No"] = p.tckn
            personel["Ad"] = p.ad
            personel["Soyad"] = p.soyad
            personel["Birim"] = p.birim.name
            personel["Görev Süresi Başlangıç"] = p.gorev_suresi_baslama.strftime("%d.%m.%Y")
            personel["Görev Süresi Bitiş"] = p.gorev_suresi_bitis.strftime("%d.%m.%Y")
            personel["Göreve Başlama Tarihi"] = p.goreve_baslama_tarihi.strftime("%d.%m.%Y")
            personel_list.append(personel)

        return personel_list



# -*- coding: utf-8 -*-

# Copyright (C) 2015 ZetaOps Inc.
#
# This file is licensed under the GNU General Public License v3
# (GPLv3).  See LICENSE.txt for details.

from ulakbus.services.ulakbus_service import UlakbusService
from zato.common import DATA_FORMAT
import os
import urllib2
import socket
from json import loads, dumps
from six import iteritems
from ulakbus.models.personel import Personel

"""HITAP Senkronizasyon Servisi

Hitap senkronizasyon servislerinin kalıtılacağı
abstract HITAP Sorgula servisini içeren modül.

Bu servis şu işlemleri gerçekleştirir:

    * Hitap'taki kayıt yerelde yoksa, yerele kaydedilir.
    * Yereldeki kayıt Hitap'ta yoksa ve sync değeri 1 (senkronize) görünüyorsa,
      Hitap'ta bu kayıt silinmiş demektir ve yerelde de silinir.

``sync`` alanı şu değerlerde bulunabilir:

    * 1: Kayıt Hitap ile senkronize
    * 2: Yerel kayıt güncellendi, Hitap güncellenecek
    * 3: Yerel kayıt silindi, Hitap kaydı silinecek
    * 4: Yeni bir yerel kayıt oluşturuldu, Hitap'a gönderilecek.


Attributes:
    H_USER (str): Hitap kullanıcı adı
    H_PASS (str): Hitap kullanıcı şifresi


Example:
    Servise JSON nesnesi kullanılarak istek gönderilmesi:

    .. code-block:: json

        $ curl http://localhost:11223/hizmet-okul-sync -d '{"tckn": "tckn"}'

"""

H_USER = os.environ["HITAP_USER"]
H_PASS = os.environ["HITAP_PASS"]


class HITAPSync(UlakbusService):
    """
    Hitap Sync servislerinin kalıtılacağı abstract Zato servisi.

    Senkronizasyon servisleri gerekli girdileri (Hitap username, Hitap password, tckn)
    Hitap sorgulama servislerine gönderip dönecek cevaba göre
    yereldeki kayıtları güncelleyebilmektedirler.

    Attributes:
        sorgula_service (str): İlgili Hitap sorgu servisinin adı
        model (Model): Hitap'taki kaydın karşılığı olan Model

    """
    HAS_CHANNEL = False

    def __init__(self):
        self.sorgula_service = ''
        self.model = None
        super(HITAPSync, self).__init__()

    def handle(self):
        """
        Servis çağrıldığında tetiklenen metod.

        Servise gelen istekten kimlik numarası (tckn) bilgisini alır ve
        Hitap sorgulama servisine gidecek isteği hazırlayacak
        ve gelen cevabı elde edecek olan sync_hitap_data fonksiyonunu çağırır.

        """

        self.logger.info("zato service started to work.")
        tckn = self.request.payload['tckn']
        self.personel = Personel.objects.filter(tckn=tckn)[0]

        self.sync_hitap_data(tckn)

    def get_hitap_dict(self, tckn):
        """
        İlgili servise ait sorgulama servisini çağırarak
        gerekli Hitap verisini elde eder.

        Args:
            tckn (str): Türkiye Cumhuriyeti Kimlik Numarası

        Returns:
            (List[dict], Bool):
                Hitap verisini yerele uygun biçimde tutan sözlük listesi ve
                Sorgu sonucunda hata olup olmadığı bilgisi

        """

        response = self.invoke(self.sorgula_service, dumps({'tckn': tckn}),
                               data_format=DATA_FORMAT.JSON, as_bunch=True)

        has_error = True if response["status"] == 'error' else False
        hitap_dict = loads(response["result"])

        return hitap_dict, has_error

    def save_hitap_data_db(self, hitap_data):
        """
        Hitap servisinden gelen kayıt yerelde yoksa, sync değeri 1 (senkronize)
        olacak şekilde kaydı veritabanına kaydeder.

        Args:
            hitap_data (dict): Hitap verisini yerele uygun biçimde tutan sözlük

        """

        obj = self.model()
        for hk, hv in iteritems(hitap_data):
            setattr(obj, hk, hv)

        obj.sync = 1
        obj.personel = self.personel
        obj.save()
        self.logger.info("hitaptaki kayit yerele kaydedildi. kayit no => "
                         + str(obj.kayit_no))

    def delete_hitap_data_db(self, kayit_no):
        """
        Yereldeki kayıt Hitap'ta yoksa ve sync değeri 1 (senkronize) görünüyorsa,
        Hitap'ta bu kayıt silinmiş demektir ve yerelde de silinir.

        Args:
            kayit_no (str): Hitap kaydının kayıt numarası

        """

        obj = self.model.objects.get(kayit_no=kayit_no)

        if obj.sync == 1:
            obj.delete()
            self.logger.info("yereldeki sync kayit hitapta yok, kayit silindi."
                             "kayit no => " + str(obj.kayit_no))

    def sync_hitap_data(self, tckn):
        """
        Yereldeki kayıtları Hitap servisinden gelen kayıtlara göre senkronize eder.
        tckn bilgisine göre Hitap kayıtları ve yereldeki kayıtları getirilir ve:

            * Hitap'taki kayıt yerelde yoksa, yerele kaydedilir.
            * Yereldeki kayıt Hitap'ta yoksa ve sync değeri 1 (senkronize) görünüyorsa,
              Hitap'ta bu kayıt silinmiş demektir ve yerelde de silinir.

        Args:
            tckn (str): Türkiye Cumhuriyeti Kimlik Numarası

        Raises:
            AttributeError: İlgili servis veya bean Hitap'ta bulunmayabilir.
            urllib2.URLError: Servis yanıt vermiyor.
            socket.error: Riak bağlantı hatası.
            Exception: Beklenmeyen hata.

        """

        # get hitap data
        hitap_dict, has_error = self.get_hitap_dict(tckn)

        if has_error:
            self.logger.info("Hitap kaydi sorgulama hatasi.")
            status = "error"
            result = "Service unavailable!"
        else:
            try:
                # get kayit no list from db
                kayit_no_list = [record.kayit_no for record in
                                 self.model.objects.filter(tckn=tckn)]
                self.logger.info("yereldeki kayit sayisi: " + str(len(kayit_no_list)))
                self.logger.info("hitaptan gelen kayit sayisi: " + str(len(hitap_dict)))

                # compare hitap data with db
                for hitap_record in hitap_dict:
                    hitap_kayit_no = hitap_record['kayit_no']

                    # if hitap data is not in db, create an object and save to db.
                    if hitap_kayit_no not in kayit_no_list:
                        self.save_hitap_data_db(hitap_record)
                    # if in db, don't touch.
                    else:
                        kayit_no_list.remove(hitap_kayit_no)

                # if there are still some in sync records which are not in hitap, delete them.
                for model_kayit_no in kayit_no_list:
                    self.delete_hitap_data_db(model_kayit_no)

                status = "ok"
                result = "Synchronisation completed successfully."

            except AttributeError:
                self.logger.exception("AttributeError")
                status = "error"
                result = "AttributeError"
            except socket.error:
                self.logger.exception("Riak connection refused!")
                status = "error"
                result = "Riak connection refused!"
            except urllib2.URLError:
                self.logger.exception("Service unavailable!")
                status = "error"
                result = "Service unavailable!"
            except Exception:
                self.logger.exception("Unexpected error!")
                status = "error"
                result = "Unexpected error!"

        self.logger.info(result)
        self.response.payload = {'status': status, 'result': result}



# -*-  coding: utf-8 -*-

# Copyright (C) 2015 ZetaOps Inc.
#
# This file is licensed under the GNU General Public License v3
# (GPLv3).  See LICENSE.txt for details.

"""HITAP Kurs Ekle

Hitap'a personelin kurs bilgilerinin eklenmesini yapar.

"""

from ulakbus.services.personel.hitap.hitap_ekle import HITAPEkle


class HizmetKursEkle(HITAPEkle):
    """
    HITAP Ekleme servisinden kalıtılmış Kurs Bilgisi Ekleme servisi

    """
    HAS_CHANNEL = True

    def handle(self):
        """Servis çağrıldığında tetiklenen metod.

        Attributes:
            service_name (str): İlgili Hitap sorgu servisinin adı
            service_dict (dict): Request yoluyla gelen kayıtlar,
                    HizmetKursInsert servisinin alanlarıyla eşlenmektedir.
                    Filtreden geçecek tarih alanları ve servis tarafında gerekli olan
                    alanlar listede tutulmaktadır.

        """

        self.service_name = 'HizmetKursInsert'
        self.service_dict = {
            'fields': {
                'tckn': self.request.payload.get('tckn', ''),
                'kursOgrenimSuresi': self.request.payload.get('kurs_ogrenim_suresi', ''),
                'mezuniyetTarihi': self.request.payload.get('mezuniyet_tarihi', ''),
                'kursNevi': self.request.payload.get('kurs_nevi', ''),
                'bolumAd': self.request.payload.get('bolum_ad', ''),
                'okulAd': self.request.payload.get('okul_ad', ''),
                'ogrenimYeri': self.request.payload.get('ogrenim_yeri', ''),
                'denklikTarihi': self.request.payload.get('denklik_tarihi', ''),
                'denklikOkul': self.request.payload.get('denklik_okulu', ''),
                'denklikBolum': self.request.payload.get('denklik_bolum', ''),
                'kurumOnayTarihi': self.request.payload.get('kurum_onay_tarihi', '')
            },
            'date_filter': ['mezuniyetTarihi', 'denklikTarihi', 'kurumOnayTarihi'],
            'required_fields': ['tckn', 'kursOgrenimSuresi', 'mezuniyetTarihi', 'kursNevi',
                                'okulAd', 'kurumOnayTarihi']
        }
        super(HizmetKursEkle, self).handle()



# -*-  coding: utf-8 -*-
# Copyright (C) 2015 ZetaOps Inc.
#
# This file is licensed under the GNU General Public License v3
# (GPLv3).  See LICENSE.txt for details.
#
from ulakbus.lib.date_time_helper import map_etkinlik_hafta_gunleri
from zengine.forms import JsonForm
from zengine.views.crud import CrudView
from collections import OrderedDict
from ulakbus.models import Personel, Donem, Ogrenci
from ulakbus.models.ders_sinav_programi import HAFTA
from ...lib.ogrenci import aktif_sinav_listesi
from zengine.lib.translation import gettext as _, get_day_names


class SinavProgramiGoruntule(CrudView):
    def sinav_etkinlikleri(self, obj):
        sinav_etkinlikleri = aktif_sinav_listesi(obj)

        if len(sinav_etkinlikleri) > 0:
            self.current.task_data['sinav_kontrol'] = True
            self.current.task_data['sinav_etkinlikleri'] = map_etkinlik_hafta_gunleri(
                sinav_etkinlikleri)
        else:
            self.current.task_data['sinav_kontrol'] = False

    def ogrenci_sinav_listesi(self):
        """
        Öğretim görevlisinin yayınlanmış sınav programının olup olmadığını kontrol eder.
        """

        # Giriş yapılan öğretim görevlisinin personel objesi getirilir.
        ogrenci = Ogrenci.objects.get(user=self.current.user)
        self.current.task_data['user_ad'] = ogrenci.__unicode__()
        self.sinav_etkinlikleri(ogrenci)

    def okutman_sinav_listesi(self):
        """
        Öğretim görevlisinin yayınlanmış sınav programının olup olmadığını kontrol eder.
        """

        # Giriş yapılan öğretim görevlisinin personel objesi getirilir.
        personel = Personel.objects.get(user=self.current.user)
        okutman = personel.okutman
        self.current.task_data['user_ad'] = okutman.__unicode__()
        self.sinav_etkinlikleri(okutman)

    def sinav_programi_uyari(self):

        """
        Eğer yayınlanmış sınav programı yoksa uyarı verir.
        """
        self.current.output['msgbox'] = {
            'type': 'info', "title": _(u'Uyarı!'),
            "msg": _(u'Bulunduğunuz döneme ait, güncel yayınlanmış sınav programı bulunmamaktadır.')
        }

    def sinav_programi_goruntule(self):
        """
        Öğretim Görevlisi kendi şubelerine ait sinav
        programını görüntüleyebilir.

        """

        sinav_etkinlikleri = self.current.task_data['sinav_etkinlikleri']

        self.output['objects'] = [list(get_day_names().values())]
        hafta = dict(HAFTA)

        # Öğretim görevlisinin bir günde maksimum kaç tane sınavı olduğu bulunur
        # ve bu bilgi kadar dönülür.
        max_etkinlik = max(map(len, sinav_etkinlikleri.values()))

        for i in range(max_etkinlik):
            sinav_etkinlik_list = OrderedDict({})

            # eğer haftanın günü(1,2..) öğretim görevlisinin sınavı varsa
            for hafta_gun in hafta.keys():
                if hafta_gun in sinav_etkinlikleri:
                    try:
                        etkinlik = sinav_etkinlikleri[hafta_gun][i]
                        sinav_etkinlik_list[hafta[hafta_gun]] = etkinlik
                    except IndexError:
                        sinav_etkinlik_list[hafta[hafta_gun]] = ''

                else:
                    sinav_etkinlik_list[hafta[hafta_gun]] = ''

            item = {
                "type": "table-multiRow",
                "fields": sinav_etkinlik_list,
                "actions": False,
                'key': ''
            }
            self.output['objects'].append(item)

        _form = JsonForm(current=self.current)
        _form.title = _(u"%(ad)s / %(donem)s / Yarıyıl Sınav Programı") % \
                      {'ad': self.current.task_data['user_ad'],
                       'donem': Donem.guncel_donem(self.current).ad}

        self.form_out(_form)



# -*-  coding: utf-8 -*-

# Copyright (C) 2015 ZetaOps Inc.
#
# This file is licensed under the GNU General Public License v3
# (GPLv3).  See LICENSE.txt for details.

"""HITAP Nüfus Sorgula

Hitap üzerinden personelin nüfus bilgilerinin sorgulamasını yapar.

Note:
    Bu servis, service ve bean isimlerindeki hatadan dolayı çalışmamaktadır.
    Açıklama için ilgili birimlere başvuruldu, yanıt bekleniyor.

"""

from ulakbus.services.personel.hitap.hitap_sorgula import HITAPSorgula


class HizmetNufusGetir(HITAPSorgula):
    """
    HITAP Sorgulama servisinden kalıtılmış Nüfus Bilgisi Sorgulama servisi

    """
    HAS_CHANNEL = True

    def handle(self):
        """
        Servis çağrıldığında tetiklenen metod.

        Attributes:
            service_name (str): İlgili Hitap sorgu servisinin adı
            bean_name (str): Hitap'tan gelen bean nesnesinin adı
            service_dict (dict): Hitap servisinden gelen kayıtların alanları,
                    ``NufusKayitlari`` modelinin alanlarıyla eşlenmektedir.
                    Filtreden geçecek tarih alanları listede tutulmaktadır.
                    Servis tarafında gerekli olan alanlar listede tutulmaktadır.

        """

        self.service_name = 'HizmetNufusSorgula'
        self.bean_name = 'HizmetNufusServisBean'
        self.service_dict = {
            'fields': {
                'tckn': 'tckn',
                'ad': 'ad',
                'soyad': 'soyad',
                'ilk_soy_ad': 'ilkSoyAd',
                'dogum_tarihi': 'dogumTarihi',
                'cinsiyet': 'cinsiyet',
                'emekli_sicil_no': 'emekliSicilNo',
                'memuriyet_baslama_tarihi': 'memuriyetBaslamaTarihi',
                'kurum_sicil': 'kurumSicili',
                'maluliyet_kod': 'maluliyetKod',
                'yetki_seviyesi': 'yetkiSeviyesi',
                'aciklama': 'aciklama',
                'kuruma_baslama_tarihi': 'kurumaBaslamaTarihi',
                'gorev_tarihi_6495': 'gorevTarihi6495',
                'emekli_sicil_6495': 'emekliSicil6495',
                'durum': 'durum',
                'sebep': 'sebep'
            },
            'date_filter': ['dogum_tarihi', 'memuriyet_baslama_tarihi', 'kuruma_baslama_tarihi'],
            'required_fields': ['tckn']
        }
        super(HizmetNufusGetir, self).handle()




#!/usr/bin/env python
# -*-  coding: utf-8 -*-
"""
"""

# Copyright (C) 2015 ZetaOps Inc.
#
# This file is licensed under the GNU General Public License v3
# (GPLv3).  See LICENSE.txt for details.


from ulakbus.lib.fake import *


class CreateUser(Command):
    CMD_NAME = 'create_user'
    HELP = 'Creates a new user'
    PARAMS = [
        {'name': 'username', 'required': True, 'help': 'Login username'},
        {'name': 'password', 'required': True, 'help': 'Login password'},
        {'name': 'abstract_role', 'default': 'BaseAbsRole', 'help': 'Name of the AbstractRole'},
        {'name': 'super', 'action': 'store_true', 'help': 'This is a super user'},
        {'name': 'permission_query', 'default': "code:crud* OR code:login* OR code:logout*",
         'help': 'Permissions which will be returned from this query will be granted to the user. '
                 'Defaults to: "code:crud* OR code:login* OR code:logout*"'},
    ]

    def run(self):
        from ulakbus.models import AbstractRole, User, Role, Permission
        if User.objects.filter(username=self.manager.args.username).count():
            print("User already exists!")
            return
        abs_role, new = AbstractRole.objects.get_or_create(name=self.manager.args.abstract_role)
        user = User(username=self.manager.args.username, superuser=self.manager.args.super)
        user.set_password(self.manager.args.password)
        user.save()
        role = Role(user=user, abstract_role=abs_role)
        role.save()
        perm_list = []
        for perm in Permission.objects.raw(self.manager.args.permission_query):
            role.Permissions(permission=perm)
            perm_list.append(perm.name)
        role.save()
        user_type = 'super user' if self.manager.args.super else 'user'
        print("New %s created with these permissions: \n\n%s" % (user_type, "\n".join(perm_list)))


class LoadFixture(Command):
    CMD_NAME = 'load_fixture'
    HELP = 'Load fixtures from given json file or files in given directory and ' \
           'dumps into ulakbus_settings_fixtures bucket, overwriting data of all existing keys.'
    PARAMS = [
        {'name': 'path', 'required': True, 'help': 'Load fixture file or directory'},
    ]

    def run(self):
        from pyoko.db.connection import client
        import os

        fixture_bucket = client.bucket_type('catalog').bucket('ulakbus_settings_fixtures')
        path = self.manager.args.path

        if os.path.isdir(path):
            from glob import glob
            for fixture_file in glob(os.path.join(path, "*.json")):
                self.dump(fixture_file, fixture_bucket)
        else:
            self.dump(path, fixture_bucket)

    @staticmethod
    def dump(fixture_file, fixture_bucket):
        try:
            with open(fixture_file) as f:
                import json
                try:
                    fixtures = json.load(f)
                    for fix in fixtures:
                        f = fixture_bucket.get(fix)
                        f.data = fixtures[fix]
                        print("%s: %s stored.." % (fix, fixtures[fix]))
                        f.store()
                except ValueError as e:
                    print("please validate your json file: %s" % e)
        except IOError:
            print("file not found: %s" % fixture_file)


class GenerateBuildingList(Command):
    CMD_NAME = 'generate_buildings'
    HELP = 'Generates fake Building model objects from Unit Faculties'
    PARAMS = []

    def run(self):
        from tests.fake.fake_data_generator import FakeDataGenerator
        fake = FakeDataGenerator()
        bina_list, room_list = fake.yeni_bina()
        print("%s adet bina oluşturuldu, oluşturulan binalar: %s\n" % (len(bina_list), bina_list))
        print("%s adet oda oluşturuldu, oluşturulan odalar: %s\n" % (len(room_list), room_list))


class GenerateRandomPersonel(Command):
    CMD_NAME = 'random_personel'
    HELP = 'Generates Random Personel'
    PARAMS = [

        {'name': 'length', 'required': False, 'help': 'Amount of random personel', 'default': 1},

    ]

    def run(self):
        from tests.fake.fake_data_generator import FakeDataGenerator
        length = int(self.manager.args.length)
        fake = FakeDataGenerator()
        fake.yeni_personel(personel_say=length)


class GenerateRandomOkutman(Command):
    CMD_NAME = 'random_okutman'
    HELP = 'Generates Random Okutman From Personel Objects'
    PARAMS = [
        {'name': 'personel', 'required': True, 'help': 'Personel object'},
    ]

    def run(self):
        from tests.fake.fake_data_generator import FakeDataGenerator
        personel = self.manager.args.personel
        fake = FakeDataGenerator()
        fake.yeni_okutman(personel=personel)


class GenerateRandomHariciOkutman(Command):
    CMD_NAME = 'random_harici_okutman'
    HELP = 'Generates Random Okutman From Personel Objects'
    PARAMS = [

        {'name': 'length', 'required': True,
         'help': 'Amount of random harici okutman', 'default': 1},

    ]

    def run(self):
        from tests.fake.fake_data_generator import FakeDataGenerator
        length = int(self.manager.args.length)
        fake = FakeDataGenerator()
        fake.yeni_harici_okutman(harici_okutman_say=length)


class GenerateRandomOgrenci(Command):
    CMD_NAME = 'random_ogrenci'
    HELP = 'Generates Random Ogrenci Model Objects'
    PARAMS = [
        {'name': 'length', 'required': False, 'help': 'Amount of random ogrenci', 'default': 1},
    ]

    def run(self):
        from tests.fake.fake_data_generator import FakeDataGenerator
        length = int(self.manager.args.length)
        fake = FakeDataGenerator()
        ogrenci_list = fake.yeni_ogrenci(ogrenci_say=length)
        print("""Toplam %s adet ogrenci oluşturuldu, oluşturulan öğrenci listesi:
                  %s""" % (length, ogrenci_list))


class GenerateProgramList(Command):
    CMD_NAME = 'generate_programs'
    HELP = 'Generates Program Records From Unit Object'
    PARAMS = [
        {'name': 'length', 'required': False, 'help': 'Amount of random program', 'default': 1},
        {'name': 'yoksis_program', 'required': True, 'help': 'Unit object'},
    ]

    def run(self):
        from tests.fake.fake_data_generator import FakeDataGenerator
        length = int(self.manager.args.length)
        yoksis_program = self.manager.args.yoksis_program
        fake = FakeDataGenerator()
        fake.yeni_program(yoksis_program=yoksis_program, program_say=length)


class GenerateDersList(Command):
    CMD_NAME = 'generate_ders'
    HELP = 'Generates fake Ders model objects'
    PARAMS = [
        {'name': 'length', 'required': False, 'help': 'Amount of random program', 'default': 1},
        {'name': 'program', 'required': True, 'help': 'Program object'},
        {'name': 'personel', 'required': True, 'help': 'Personel object'},
    ]

    def run(self):
        from tests.fake.fake_data_generator import FakeDataGenerator
        length = int(self.manager.args.length)
        personel = self.manager.args.personel
        program = self.manager.args.program
        fake = FakeDataGenerator()
        fake.yeni_ders(program=program, personel=personel, ders_say=length)


class DeployZatoServices(Command):
    CMD_NAME = 'load_zato_services'
    HELP = """Read --path, discover services and load them into Database
    as Service Files and Channels."""
    PARAMS = [
        {'name': 'path', 'required': True,
         'help': 'The zato service file or folder to be installed.'},
    ]
    py_files_path = []

    def get_py_files_path(self, path):
        """
        servicelerin bulundugu klasorlerde arama yaparak .py dosyalarini bulur.
        Args:
            path: management komutunun --path parametresinden gelir

        Returns: path parametresine e gore servis veya servislerin path uzantilarini
        py_files_path dizisine ekler.

        """
        from glob import glob

        for g in glob(path + '/*'):
            file_name_extension = os.path.splitext(os.path.basename(g))[1]
            if file_name_extension == '.py':
                self.py_files_path.append(g)
            elif not file_name_extension:
                self.get_py_files_path(g)

    def run(self):
        import importlib
        import inspect

        try:
            from zato.server.service import Service
        except ImportError:
            print ("""Please install zato fake lib:
                      pip install git+https://github.com/zetaops/fake_zato.git""")
            raise

        path = self.manager.args.path
        if os.path.splitext(os.path.basename(path))[1] == '.py':
            paths = [path]
        else:
            self.get_py_files_path(path)
            paths = self.py_files_path

        for p in paths:
            bool_create_service_object = False
            name = os.path.splitext(os.path.basename(p))[0]
            # Ignore __ files
            if name.startswith("__"):
                continue
            pth = os.path.splitext(p)[0]
            module_name = '.'.join(pth.split('/'))
            module = importlib.import_module(module_name)
            for obj_name in dir(module):
                if obj_name.startswith("__"):
                    continue
                obj = getattr(module, obj_name)
                # Eger obj bir class ise, zato `Service`  inherited ise,
                #  ve Service in kendisi degilse
                if inspect.isclass(obj) and issubclass(obj, Service) and hasattr(obj,
                                                                                 "HAS_CHANNEL"):

                    if not bool_create_service_object:
                        self.create_service_file_object(p)
                        bool_create_service_object = True

                    if obj.HAS_CHANNEL:
                        self.create_channel_and_service_object(obj, p)

    @staticmethod
    def create_channel_and_service_object(service, path):
        from ulakbus.models.zato import ZatoServiceChannel
        from pyoko.lib.utils import un_camel

        create_channel = ZatoServiceChannel()
        create_channel.cluster_id = 1
        create_channel.service_name = service.get_name()
        create_channel.channel_name = service.get_name() + "-channel"
        create_channel.channel_url_path = '/%s/%s' % ('/'.join(path.split('/')[:-1]),
                                                      un_camel(service.__name__, dash='-'))
        create_channel.channel_connection = "channel"
        create_channel.channel_data_format = "json"
        create_channel.channel_is_internal = False
        create_channel.channel_is_active = True
        create_channel.channel_transport = "plain_http"
        create_channel.save()

    def create_service_file_object(self, path):
        from ulakbus.models.zato import ZatoServiceFile

        service_payload = self.get_service_payload_data(path)

        upload_service = ZatoServiceFile()
        upload_service.cluster_id = 1
        upload_service.service_payload_name = service_payload["payload_name"]
        upload_service.service_payload = service_payload["payload"]
        upload_service.save()

    @staticmethod
    def get_service_payload_data(path):
        import base64

        with open(path, 'r') as f:
            text = f.read()
            payload = base64.b64encode(text)

        service_payload = dict()
        service_payload["payload_name"] = os.path.basename(path)
        service_payload["payload"] = payload

        return service_payload

environ['PYOKO_SETTINGS'] = 'ulakbus.settings'
environ['ZENGINE_SETTINGS'] = 'ulakbus.settings'

if __name__ == '__main__':
    ManagementCommands()



# -*-  coding: utf-8 -*-

# Copyright (C) 2015 ZetaOps Inc.
#
# This file is licensed under the GNU General Public License v3
# (GPLv3).  See LICENSE.txt for details.

"""Banka Service

Banka servislerinin kalıtılacağı abstract Banka servisini ve
banka yetkilendirme kontrolü fonksiyonunu içeren modül.

"""

from ulakbus.models.ogrenci import Banka, BankaAuth
from ulakbus.services.ulakbus_service import UlakbusService


class AuthException(Exception):
    """
    Banka yetkilendirmesi için verilen bilgilerin sistemdeki bilgilerle
    uyuşmaması durumunda fırlatılan Yetkilendirme Hatası

    """

    pass


def authenticate(func):
    """
    Banka yetkilendirme kontrolü için decorator fonksiyon.

    Banka servislerine istek geldiğinde öncelikle bu fonksiyon tetiklenir ve
    Banka kullanıcı bilgileri kontrol edilir.

    Bilgiler sistemdeki bilgilerle uyuşuyorsa işleme devam edilir.
    Aksi taktirde AuthException hatası verilir.

    Args:
        func (function): Yetkilendirme işleminden geçmesi gereken fonksiyon.

    """

    def auth(self):
        try:
            self.banka = Banka.objects.get(kod=str(self.request.input.banka_kodu))
            BankaAuth.objects.get(username=self.request.input.bank_username,
                                  password=self.request.input.bank_password,
                                  banka=self.banka)
            self.logger.info("Authentication completed successfully.")
        except Exception as e:
            raise AuthException("Authentication failed. %s" % e)
        return func(self)

    return auth


class BankaService(UlakbusService):
    """
    Banka servislerinin kalıtılacağı abstract Zato Servisi.

    Banka servisleri bu servisle yetkilendirme işleminden geçip
    gerekli girdileri elde eder ve çıktıları geri döndürürler.

    Note:
        self.request (zato.server.service.Request):
            Servise gelen isteği tutan Zato nesnesi.
            Girdileri elde etmek için `self.request.input` kullanılır.
        self.response (zato.server.service.Response):
            Servisten dönecek olan cevabı tutan Zato nesnesi.
            Çıktı parametrelerini saklamak için `self.response.payload` kullanılır.

    """

    HAS_CHANNEL = False

    def __init__(self):
        #: Banka: Servisleri kullanacak olan yetkilendirilecek banka.
        self.banka = None
        super(BankaService, self).__init__()

    class SimpleIO:
        """
        Servis girdilerinin ve çıktılarının belirlendiği yapı.

        Servis çağrısı yapılırken:
            - request_elem: Servise gelen isteğin (JSON, XML) ismi
            - input_required: Zorunlu olarak sağalanacak girdilerin listesi
            - input_optional: İsteğe bağlı olarak sağlanacak girdilerin listesi
            - default_value: İsteğe bağlı girdilerin değerlerinin, servis çağrılırken
              verilmemesi durumunda kullanılacak varsayılan değer

        Servisin döneceği cevap belirlenirken:
            - response_elem: Servisten dönen veriyi içeren (payload)
              cevabın (JSON, XML) ismi
            - output_required: Zorunlu olarak döndürülerek çıktıların listesi
            - output_optional: İsteğe bağlı olarak döndürülecek çıktıların listesi

        Note:
            SimpleIO sınıfında belirlenen özellikler,
            doğrudan yer aldığı servisin özellikleri olmaktadır.

            - banka_kodu (str): Üniversite tarafından bankaya verilen kod
            - bank_username (str): Üniversite tarafından bankaya verilen kullanıcı adı
            - bank_password (str): Üniversite tarafından bankaya verilen şifre

        """

        input_required = ('banka_kodu', 'bank_username', 'bank_password')
        output_required = ()

    def handle(self):
        """
        Servis çağrıldığında tetiklenen metod.

        Yetkilendirme işleminin sonucuna göre servis çalışmaya devam eder.

        """

        try:
            self.get_data()
        except AuthException:
            self.logger.exception("Authentication failed.")

    @authenticate
    def get_data(self):
        """
        Servise gelen girdileri alan ve bu girdilere göre gerekli bilgileri
        elde edip cevap olarak döndüren metod.

        Gerçekleştirimi bu servisten kalıtılan servisler tarafından yapılmaktadır.

        """

        pass



# -*-  coding: utf-8 -*-
"""
"""

# Copyright (C) 2015 ZetaOps Inc.
#
# This file is licensed under the GNU General Public License v3
# (GPLv3).  See LICENSE.txt for details.
#
# Personele ait hizmet kayıtlarının gösterilmesini ve yazdırılmasını içeren WF
from ulakbus.models.personel import Personel
from ulakbus.views.personel.crud_hitap import zato_service_selector
from ulakbus.views.reports.base import Reporter
from zengine.forms import fields
from ulakbus.models.hitap.hitap import HizmetKayitlari
from collections import OrderedDict
import datetime

from zengine.views.crud import view_method


class ReportWithSync(Reporter.ReportForm):
    """
    ReportForm Sync eklenmis versiyonu
    """
    printout = fields.Button("Yazdır", cmd="printout")
    sync = fields.Button("HITAP ile senkronize et", cmd="sync")


class HizmetCetveli(Reporter):
    TITLE = "Hizmet Cetveli Listesi"

    def __init__(self, current=None):
        self.ReportForm = ReportWithSync
        super(HizmetCetveli, self).__init__(current)

        if self.cmd == 'sync':
            self.sync()

    def get_objects(self):
        """
        Personel id ile kayıtları filtreleyip sıralı bir şekilde listeler
        :return:
        """
        if 'id' in self.input:
            self.current.task_data['personel_id'] = self.input['id']
            self.current.task_data['personel_tckn'] = Personel.objects.get(self.input['id']).tckn

        hk_list = []
        for hk in HizmetKayitlari.objects.filter(
                personel_id=self.current.task_data['personel_id']).order_by('order_date'):
            hk_record = OrderedDict({})
            hk_record['Görev'] = hk.gorev.title() if hk.gorev else ''
            hk_record['H. Sınıf'] = hk.get_hizmet_sinifi_display()
            hk_record['Kadro'] = str(hk.kadro_derece)
            hk_record['GA'] = "%i/%i" % (hk.odeme_derece, hk.odeme_kademe)
            hk_record['KH'] = "%i/%i" % (
                hk.kazanilmis_hak_ayligi_derece, hk.kazanilmis_hak_ayligi_kademe)
            hk_record['EM'] = "%i/%i" % (hk.emekli_derece, hk.emekli_kademe)
            hk_record['Başlangıç'] = str(
                hk.baslama_tarihi) if hk.baslama_tarihi != datetime.date(1900, 1, 1) else ''

            hk_record['Bitiş'] = str(
                hk.bitis_tarihi) if hk.bitis_tarihi != datetime.date(1900, 1, 1) else ''

            hk_record['Sebep'] = str(hk.sebep_kod)
            hk_list.append(hk_record)
        return hk_list

    def sync(self):
        """
        HizmetKayıtları için Hitap sync fonksiyonunu çalıştırır
        :return:
        """
        hitap_service = zato_service_selector(HizmetKayitlari, 'sync')
        hs = hitap_service(tckn=str(self.current.task_data['personel_tckn']))
        hs.zato_request()

    def clear_cmd(self):
        """
        sync sonrası sonsuz döngüye girmesini engellemek için cmd show olarak değiştirildi
        :return:
        """
        self.current.input['cmd'] = 'show'



# -*-  coding: utf-8 -*-
"""
"""

# Copyright (C) 2015 ZetaOps Inc.
#
# This file is licensed under the GNU General Public License v3
# (GPLv3).  See LICENSE.txt for details.
from collections import defaultdict
from datetime import timedelta, date
import calendar
from zengine.lib.translation import gettext_lazy, gettext as _, LazyProxy, get_day_names, get_month_names

__author__ = 'Ali Riza Keles'


def gun_dilimi_listele():
    return [
        (1, _(u'Sabah')),
        (2, _(u'Öğle')),
        (3, _(u'Akşam')),
    ]


def _liste_hazirla(fn, make_choices=False):
    """Babel'ın `get_day_names` ve `get_month_names` özelliklerini Ulakbus'e uyarlar.

    Babel'ın `get_day_names` ve `get_month_names` methodları, gün ve ayları 0-indeksli
    sözlükler olarak veriyor. Ancak uygulamanın bir çok yerinde `HAFTA` ve `AYLAR`
    listelerinin 1-indeksli tuple'lar içeren bir liste olması bekleniyor. Ayrıca, gün
    ve ay isimlerinin bakılmasının, isimler kullanıcıya gönderilmeden hemen önce
    yapılması gerekli ki kullanıcının seçtiği tarih tercihine göre gönderilebilsin.

    Bunu sağlamak adına, LazyProxy ile listenin oluşturulması kullanım anına ertelenmiş,
    yardımcı fonksiyon ile de indeksler beklenen sayılara kaydırılmıştır.
    """
    def choices_title_map():
        liste = []
        for i, eleman in fn().items():
            liste.append({'name': eleman, 'value': i})
        return liste
    if not make_choices:
        return LazyProxy(lambda: fn().items(), enable_cache=False)
    else:
        return choices_title_map


def _get_day_names():
    modified_days = {}
    days = get_day_names().items()
    for i, name in days:
        modified_days[i + 1] = name
    return modified_days


HAFTA = _liste_hazirla(_get_day_names)
gun_listele = _liste_hazirla(_get_day_names, make_choices=True)

AYLAR = _liste_hazirla(get_month_names)
ay_listele = _liste_hazirla(get_month_names, make_choices=True)


def map_etkinlik_hafta_gunleri(etkinlikler):
    """
    Ders ve Sinav Etkinliği  nesnelerini tarihlerine göre haftanın günlerine dağıtır.

    Haftanınn günlerinin key olarak kullanıldığı  bir dict üretir.

    Args:
        etkinlikler(list) : Sinav ya da ders etkinliği objeleri listesi

    Returns:
        dict
    """

    d = defaultdict(list)
    for etkinlik in etkinlikler:
        if hasattr(etkinlik, "tarih"):
            weekday = etkinlik.tarih.isoweekday()
            etkinlik_listesi = d.get(weekday, [])
            etkinlik_listesi.append(etkinlik.__unicode__())
            d[weekday] = etkinlik_listesi
        else:
            weekday = etkinlik.gun
            ders_etkinlikleri = d[weekday]
            ders_etkinlikleri.append(etkinlik.__unicode__())
            d[weekday] = ders_etkinlikleri
    return d


def zaman_araligi(baslangic, bitis):
    """
    Verilen iki tarih arasinda kalan tarihleri
    donduren method.

    Args:
        baslangic (datetime.date): Date 02.04.2016
        bitis (datetime.date): Date 04.04.2016

    Returns:
        (list) [datetime.date(2016, 4, 2), datetime.date(2016, 4, 3), datetime.date(2016, 4, 4)]

    """
    for n in range(int((bitis - baslangic).days) + 1):
        yield baslangic + timedelta(n)


def resmi_tatil_gunleri_getir(birim_unit, yil, ay):
    from ulakbus.models.ogrenci import Takvim, Donem
    from ulakbus.lib.common import get_akademik_takvim

    baslangic, bitis = yil_ve_aya_gore_ilk_ve_son_gun(yil, ay)
    resmi_tatil_list = []
    donem_list = Donem.takvim_ayina_rastlayan_donemler(yil, ay)
    for donem in donem_list:
        akademik_takvim = get_akademik_takvim(birim_unit, donem.ogretim_yili)
        tatil_list = []
        for resmi_tatil in Takvim.objects.filter(akademik_takvim=akademik_takvim,
                                                 resmi_tatil=True, baslangic__gte=baslangic,
                                                 bitis__lte=bitis):
            for gun in zaman_araligi(resmi_tatil.baslangic, resmi_tatil.bitis):
                tatil_list.append(gun.day)
        resmi_tatil_list.append(tatil_list)

    return resmi_tatil_list


def yil_ve_aya_gore_ilk_ve_son_gun(yil, ay):
    ilk = date(yil, ay, 1)
    ayin_ilk_hafta_gunu, ay_sonu = calendar.monthrange(yil, ay)
    son = date(yil, ay, ay_sonu)

    return ilk, son



# -*-  coding: utf-8 -*-

# Copyright (C) 2015 ZetaOps Inc.
#
# This file is licensed under the GNU General Public License v3
# (GPLv3).  See LICENSE.txt for details.

"""HITAP Unvan Ekle

Hitap'a personelin Unvan bilgilerinin eklenmesini yapar.

"""

from ulakbus.services.personel.hitap.hitap_ekle import HITAPEkle


class HizmetUnvanEkle(HITAPEkle):
    """
    HITAP Ekleme servisinden kalıtılmış Hizmet Unvan Bilgi Ekleme servisi

    """
    HAS_CHANNEL = True

    def handle(self):
        """Servis çağrıldığında tetiklenen metod.

        Attributes:
            service_name (str): İlgili Hitap sorgu servisinin adı
            service_dict (dict): Request yoluyla gelen kayıtlar,
                    HizmetUnvanInsert servisinin alanlarıyla eşlenmektedir.
                    Filtreden geçecek tarih alanları ve servis tarafında gerekli olan
                    alanlar listede tutulmaktadır.

        """

        self.service_name = 'HizmetUnvanInsert'
        self.service_dict = {
            'fields': {
                'asilVekil': self.request.payload.get('asil_vekil', ''),
                'atamaSekli': self.request.payload.get('atama_sekli', ''),
                'hizmetSinifi': self.request.payload.get('hizmet_sinifi', ''),
                'tckn': self.request.payload.get('tckn', ''),
                'unvanKod': self.request.payload.get('unvan_kod', ''),
                'unvanTarihi': self.request.payload.get('unvan_tarihi', ''),
                'unvanBitisTarihi': self.request.payload.get('unvan_bitis_tarihi', ''),
                'kurumOnayTarihi': self.request.payload.get('kurum_onay_tarihi', ''),
                'fhzOrani': self.request.payload.get('fhz_orani', '')
            },
            'date_filter': ['unvanTarihi', 'unvanBitisTarihi', 'kurumOnayTarihi'],
            'required_fields': ['tckn', 'unvanKod', 'unvanTarihi', 'hizmetSinifi', 'asilVekil',
                                'atamaSekli', 'kurumOnayTarihi']

        }
        super(HizmetUnvanEkle, self).handle()



# coding=utf-8
# see LICENSE file for license information

from reportlab.lib import colors
from reportlab.lib.enums import TA_RIGHT
from reportlab.lib.fonts import addMapping
from reportlab.lib.styles import getSampleStyleSheet
from reportlab.lib.units import cm, mm
from reportlab.pdfbase import pdfmetrics
from reportlab.pdfbase.ttfonts import TTFont
from reportlab.platypus import (
    BaseDocTemplate, Spacer, Frame, PageTemplate, NextPageTemplate, PageBreak,
    Table, KeepTogether, CondPageBreak, Paragraph as _Paragraph, TableStyle)
from reportlab.platypus.flowables import HRFlowable

import copy
import sys
import unicodedata


PY2 = (sys.version_info[0] < 3)

if PY2:
    string_type = unicode
else:
    string_type = str


def register_fonts_from_paths(regular, italic=None, bold=None, bolditalic=None,
                              font_name='Reporting'):
    """
    Pass paths to TTF files which should be used for the PDFDocument
    """

    pdfmetrics.registerFont(TTFont('%s' % font_name, regular))
    pdfmetrics.registerFont(TTFont('%s-Italic' % font_name, italic or regular))
    pdfmetrics.registerFont(TTFont('%s-Bold' % font_name, bold or regular))
    pdfmetrics.registerFont(TTFont('%s-BoldItalic' % font_name, bolditalic or bold or regular))

    addMapping('%s' % font_name, 0, 0, '%s' % font_name)
    addMapping('%s' % font_name, 0, 1, '%s-Italic' % font_name)
    addMapping('%s' % font_name, 1, 0, '%s-Bold' % font_name)
    addMapping('%s' % font_name, 1, 1, '%s-BoldItalic' % font_name)


class Empty(object):
    pass


def sanitize(text):
    REPLACE_MAP = [
        (u'&', '&#38;'),
        (u'<', '&#60;'),
        (u'>', '&#62;'),
        (u'ç', '&#231;'),
        (u'Ç', '&#199;'),
        (u'\n', '<br />'),
        (u'\r', ''),
    ]

    for p, q in REPLACE_MAP:
        text = text.replace(p, q)
    return text


def normalize(text):
    """
    Some layers of reportlab, PDF or font handling or whatever cannot handle
    german umlauts in decomposed form correctly. Normalize everything to
    NFKC.
    """
    if not isinstance(text, string_type):
        text = string_type(text)
    return unicodedata.normalize('NFKC', text)


def MarkupParagraph(txt, *args, **kwargs):
    if not txt:
        return _Paragraph(u'', *args, **kwargs)
    return _Paragraph(normalize(txt), *args, **kwargs)


def Paragraph(txt, *args, **kwargs):
    if not txt:
        return _Paragraph(u'', *args, **kwargs)
    return _Paragraph(sanitize(normalize(txt)), *args, **kwargs)


class BottomTable(Table):
    """
    This table will automatically be moved to the bottom of the page using the
    BottomSpacer right before it.
    """
    pass


class BottomSpacer(Spacer):
    def wrap(self, availWidth, availHeight):
        my_height = availHeight - self._doc.bottomTableHeight

        if my_height <= 0:
            return (self.width, availHeight)
        else:
            return (self.width, my_height)


class RestartPageBreak(PageBreak):
    """
    Insert a page break and restart the page numbering.
    """
    pass


class ReportingDocTemplate(BaseDocTemplate):
    def __init__(self, *args, **kwargs):
        BaseDocTemplate.__init__(self, *args, **kwargs)
        self.bottomTableHeight = 0
        self.bottomTableIsLast = False
        self.numPages = 0
        self._lastNumPages = 0
        self.setProgressCallBack(self._onProgress_cb)

        # For batch reports with several PDFs concatenated
        self.restartDoc = False
        self.restartDocIndex = 0
        self.restartDocPageNumbers = []

    def afterFlowable(self, flowable):
        self.numPages = max(self.canv.getPageNumber(), self.numPages)
        self.bottomTableIsLast = False

        if isinstance(flowable, BottomTable):
            self.bottomTableHeight = reduce(
                lambda p, q: p+q,
                flowable._rowHeights,
                0)

            self.bottomTableIsLast = True

        elif isinstance(flowable, RestartPageBreak):
            self.restartDoc = True
            self.restartDocIndex += 1
            self.restartDocPageNumbers.append(self.page)

    # here the real hackery starts ... thanks Ralph
    def _allSatisfied(self):
        """ Called by multi-build - are all cross-references resolved? """
        if self._lastNumPages < self.numPages:
            return 0
        return BaseDocTemplate._allSatisfied(self)

    def _onProgress_cb(self, what, arg):
        if what == 'STARTED':
            self._lastNumPages = self.numPages
            self.restartDocIndex = 0
            # self.restartDocPageNumbers = []

    def page_index(self):
        """
        Return the current page index as a tuple (current_page, total_pages)

        This is the ugliest thing I've done in the last two years.
        For this I'll burn in programmer hell.

        At least it is contained here.

        (Determining the total number of pages in reportlab is a mess
        anyway...)
        """

        current_page = self.page
        total_pages = self.numPages

        if self.restartDoc:
            if self.restartDocIndex:
                current_page = (
                    current_page
                    - self.restartDocPageNumbers[self.restartDocIndex - 1]
                    + 1)
                if len(self.restartDocPageNumbers) > self.restartDocIndex:
                    total_pages = (
                        self.restartDocPageNumbers[self.restartDocIndex]
                        - self.restartDocPageNumbers[self.restartDocIndex - 1]
                        + 1)
            else:
                total_pages = self.restartDocPageNumbers[0]

        if self.bottomTableHeight:
            total_pages -= 1

            if self.bottomTableIsLast and current_page == 1:
                total_pages = max(1, total_pages)

        # Ensure total pages is always at least 1
        total_pages = max(1, total_pages)

        return (current_page, total_pages)

    def page_index_string(self):
        """
        Return page index string for the footer.
        """
        current_page, total_pages = self.page_index()

        return self.PDFDocument.page_index_string(current_page, total_pages)


def dummy_stationery(c, doc):
    pass


class PDFDocument(object):
    show_boundaries = False
    _watermark = None

    def __init__(self, *args, **kwargs):
        self.doc = ReportingDocTemplate(*args, **kwargs)
        self.doc.PDFDocument = self
        self.story = []

        self.font_name = kwargs.get('font_name', 'Vera')
        self.font_size = kwargs.get('font_size', 9)

    def page_index_string(self, current_page, total_pages):
        return 'Page %(current_page)d of %(total_pages)d' % {
            'current_page': current_page,
            'total_pages': total_pages,
        }

    def generate_style(self, font_name=None, font_size=None):
        self.style = Empty()
        self.style.fontName = font_name or self.font_name
        self.style.fontSize = font_size or self.font_size

        _styles = getSampleStyleSheet()

        self.style.normal = _styles['Normal']
        self.style.normal.fontName = '%s' % self.style.fontName
        self.style.normal.fontSize = self.style.fontSize
        self.style.normal.firstLineIndent = 0
        # normal.textColor = '#0e2b58'

        self.style.heading1 = copy.deepcopy(self.style.normal)
        self.style.heading1.fontName = '%s' % self.style.fontName
        self.style.heading1.fontSize = 1.5 * self.style.fontSize
        self.style.heading1.leading = 2 * self.style.fontSize
        # heading1.leading = 10*mm

        self.style.heading2 = copy.deepcopy(self.style.normal)
        self.style.heading2.fontName = '%s-Bold' % self.style.fontName
        self.style.heading2.fontSize = 1.25 * self.style.fontSize
        self.style.heading2.leading = 1.75 * self.style.fontSize
        # heading2.leading = 5*mm

        self.style.heading3 = copy.deepcopy(self.style.normal)
        self.style.heading3.fontName = '%s-Bold' % self.style.fontName
        self.style.heading3.fontSize = 1.1 * self.style.fontSize
        self.style.heading3.leading = 1.5 * self.style.fontSize
        self.style.heading3.textColor = '#666666'
        # heading3.leading = 5*mm

        self.style.small = copy.deepcopy(self.style.normal)
        self.style.small.fontSize = self.style.fontSize - 0.9

        self.style.smaller = copy.deepcopy(self.style.normal)
        self.style.smaller.fontSize = self.style.fontSize * 0.75

        self.style.bold = copy.deepcopy(self.style.normal)
        self.style.bold.fontName = '%s-Bold' % self.style.fontName

        self.style.boldr = copy.deepcopy(self.style.bold)
        self.style.boldr.alignment = TA_RIGHT

        self.style.right = copy.deepcopy(self.style.normal)
        self.style.right.alignment = TA_RIGHT

        self.style.indented = copy.deepcopy(self.style.normal)
        self.style.indented.leftIndent = 0.5*cm

        self.style.tablenotes = copy.deepcopy(self.style.indented)
        self.style.tablenotes.fontName = '%s-Italic' % self.style.fontName

        self.style.paragraph = copy.deepcopy(self.style.normal)
        self.style.paragraph.spaceBefore = 1
        self.style.paragraph.spaceAfter = 1

        self.style.bullet = copy.deepcopy(self.style.normal)
        self.style.bullet.bulletFontName = 'Symbol'
        self.style.bullet.bulletFontSize = 7
        self.style.bullet.bulletIndent = 6
        self.style.bullet.firstLineIndent = 0
        self.style.bullet.leftIndent = 15

        self.style.numberbullet = copy.deepcopy(self.style.normal)
        self.style.numberbullet.bulletFontName = self.style.paragraph.fontName
        self.style.numberbullet.bulletFontSize = self.style.paragraph.fontSize
        self.style.numberbullet.bulletIndent = 0
        self.style.numberbullet.firstLineIndent = 0
        self.style.numberbullet.leftIndent = 15

        # alignment = TA_RIGHT
        # leftIndent = 0.4*cm
        # spaceBefore = 0
        # spaceAfter = 0

        self.style.tableBase = (
            (
                'FONT', (0, 0), (-1, -1),
                '%s' % self.style.fontName,  self.style.fontSize),
            ('TOPPADDING', (0, 0), (-1, -1), 0),
            ('BOTTOMPADDING', (0, 0), (-1, -1), 1),
            ('LEFTPADDING', (0, 0), (-1, -1), 0),
            ('RIGHTPADDING', (0, 0), (-1, -1), 0),
            ('FIRSTLINEINDENT', (0, 0), (-1, -1), 0),
            ('VALIGN', (0, 0), (-1, -1), 'TOP'),
        )

        self.style.table = self.style.tableBase+(
            ('ALIGN', (1, 0), (-1, -1), 'RIGHT'),
        )

        self.style.tableLLR = self.style.tableBase+(
            ('ALIGN', (2, 0), (-1, -1), 'RIGHT'),
            ('VALIGN', (0, 0), (-1, 0), 'BOTTOM'),
        )

        self.style.tableHead = self.style.tableBase+(
            (
                'FONT', (0, 0), (-1, 0),
                '%s-Bold' % self.style.fontName,  self.style.fontSize),
            ('ALIGN', (1, 0), (-1, -1), 'RIGHT'),
            ('TOPPADDING', (0, 0), (-1, -1), 1),
            ('BOTTOMPADDING', (0, 0), (-1, -1), 2),
            ('LINEABOVE', (0, 0), (-1, 0), 0.2, colors.black),
            ('LINEBELOW', (0, 0), (-1, 0), 0.2, colors.black),
        )

        self.style.tableOptional = self.style.tableBase+(
            (
                'FONT', (0, 0), (-1, 0),
                '%s-Italic' % self.style.fontName, self.style.fontSize),
            ('ALIGN', (1, 0), (-1, -1), 'RIGHT'),
            ('BOTTOMPADDING', (0, 0), (-1, -1), 5),
            ('RIGHTPADDING', (1, 0), (-1, -1), 2*cm),
        )

    def init_templates(self, page_fn, page_fn_later=None):
        self.doc.addPageTemplates([
            PageTemplate(
                id='First',
                frames=[self.frame],
                onPage=page_fn),
            PageTemplate(
                id='Later',
                frames=[self.frame],
                onPage=page_fn_later or page_fn),
        ])
        self.story.append(NextPageTemplate('Later'))

    def init_report(self, page_fn=dummy_stationery, page_fn_later=None):
        frame_kwargs = {
            'showBoundary': self.show_boundaries,
            'leftPadding': 0,
            'rightPadding': 0,
            'topPadding': 0,
            'bottomPadding': 0,
        }

        full_frame = Frame(2.6*cm, 2*cm, 16.4*cm, 25*cm, **frame_kwargs)

        self.doc.addPageTemplates([
            PageTemplate(
                id='First',
                frames=[full_frame],
                onPage=page_fn),
            PageTemplate(
                id='Later',
                frames=[full_frame],
                onPage=page_fn_later or page_fn),
        ])
        self.story.append(NextPageTemplate('Later'))

        self.generate_style(font_size=12)

    def init_confidential_report(self, page_fn=dummy_stationery,
                                 page_fn_later=None):
        if not page_fn_later:
            page_fn_later = page_fn

        def _first_page_fn(canvas, doc):
            page_fn(canvas, doc)
            doc.PDFDocument.confidential(canvas)
            doc.PDFDocument.watermark('CONFIDENTIAL')

        self.init_report(page_fn=_first_page_fn, page_fn_later=page_fn_later)

    def init_letter(self, page_fn=dummy_stationery, page_fn_later=None,
                    address_y=None, address_x=None):
        frame_kwargs = {
            'showBoundary': self.show_boundaries,
            'leftPadding': 0,
            'rightPadding': 0,
            'topPadding': 0,
            'bottomPadding': 0,
        }

        address_frame = Frame(
            address_x or 2.6 * cm,
            address_y or 20.2 * cm,
            16.4 * cm,
            4 * cm,
            **frame_kwargs
        )
        rest_frame = Frame(2.6*cm, 2*cm, 16.4*cm, 18.2*cm, **frame_kwargs)
        full_frame = Frame(2.6*cm, 2*cm, 16.4*cm, 25*cm, **frame_kwargs)

        self.doc.addPageTemplates([
            PageTemplate(
                id='First',
                frames=[address_frame, rest_frame],
                onPage=page_fn),
            PageTemplate(
                id='Later',
                frames=[full_frame],
                onPage=page_fn_later or page_fn),
        ])
        self.story.append(NextPageTemplate('Later'))

        self.generate_style(font_size=9)

    def watermark(self, watermark=None):
        self._watermark = watermark

    def restart(self):
        self.story.append(NextPageTemplate('First'))
        self.story.append(RestartPageBreak())

    def p(self, text, style=None):
        self.story.append(Paragraph(text, style or self.style.normal))

    def h1(self, text, style=None):
        self.story.append(Paragraph(text, style or self.style.heading1))

    def h2(self, text, style=None):
        self.story.append(Paragraph(text, style or self.style.heading2))

    def h3(self, text, style=None):
        self.story.append(Paragraph(text, style or self.style.heading3))

    def small(self, text, style=None):
        self.story.append(Paragraph(text, style or self.style.small))

    def smaller(self, text, style=None):
        self.story.append(Paragraph(text, style or self.style.smaller))

    def p_markup(self, text, style=None):
        self.story.append(MarkupParagraph(text, style or self.style.normal))

    def ul(self, items):
        for item in items:
            self.story.append(
                MarkupParagraph(item, self.style.bullet, bulletText=u'•'))

    def spacer(self, height=0.6*cm):
        self.story.append(Spacer(1, height))

    def table(self, data, style=None):
        table= Table(data, hAlign='LEFT', colWidths=(100*mm, None))
        table.setStyle(TableStyle([
            ('FONT', (0, 0), (-1, 0), 'Vera'),
            ('ALIGN', (10, 0), (-1, 0), 'CENTER'),
            ('ALIGN', (0, 0), (0, -1), 'LEFT'),
            ('INNERGRID', (0, 0), (-1, -1), 0.50, colors.black),
            ('BOX', (0, 0), (-1, -1), 0.25, colors.black),
        ]))
        self.story.append(table)

    def hr(self):
        self.story.append(
            HRFlowable(width='100%', thickness=0.2, color=colors.black))

    def hr_mini(self):
        self.story.append(
            HRFlowable(width='100%', thickness=0.2, color=colors.grey))

    def mini_html(self, html):
        """Convert a small subset of HTML into ReportLab paragraphs

        Requires lxml and BeautifulSoup."""
        import lxml.html
        import lxml.html.soupparser

        TAG_MAP = {
            'strong': 'b',
            'em': 'i',
            'br': 'br',  # Leave br tags alone
        }

        BULLETPOINT = u'•'

        def _p(text, list_bullet_point, style=None):
            if list_bullet_point:
                self.story.append(
                    MarkupParagraph(
                        text,
                        style or self.style.paragraph,
                        bulletText=list_bullet_point)
                )
            else:
                self.story.append(
                    MarkupParagraph(
                        text,
                        style or self.style.paragraph)
                )

        def _remove_attributes(element):
            for key in element.attrib:
                del element.attrib[key]

        def _handle_element(element, list_bullet_point=False, style=None):
            _remove_attributes(element)

            if element.tag in TAG_MAP:
                element.tag = TAG_MAP[element.tag]

            if element.tag in ('ul',):
                for item in element:
                    _handle_element(
                        item,
                        list_bullet_point=BULLETPOINT,
                        style=self.style.bullet)
                list_bullet_point = False
            elif element.tag in ('ol',):
                for counter, item in enumerate(element):
                    _handle_element(
                        item,
                        list_bullet_point=u'{}.'.format(counter + 1),
                        style=self.style.numberbullet,
                    )
                list_bullet_point = False
            elif element.tag in ('p', 'li'):
                for tag in reversed(list(element.iterdescendants())):
                    _remove_attributes(tag)
                    if tag.tag in TAG_MAP:
                        tag.tag = TAG_MAP[tag.tag]
                    else:
                        tag.drop_tag()

                _p(
                    lxml.html.tostring(
                        element, method='xml', encoding=string_type),
                    list_bullet_point,
                    style)
            else:
                if element.text:
                    _p(element.text, list_bullet_point, style)

                for item in element:
                    _handle_element(item, list_bullet_point, style)

            if element.tail:
                _p(element.tail, list_bullet_point, style)

        soup = lxml.html.soupparser.fromstring(html)
        for element in soup:
            _handle_element(element)

    def pagebreak(self):
        self.story.append(PageBreak())

    def bottom_table(self, data, columns, style=None):
        obj = BottomSpacer(1, 1)
        obj._doc = self.doc
        self.story.append(obj)

        self.story.append(
            BottomTable(data, columns, style=style or self.style.table))

    def append(self, data):
        self.story.append(data)

    def generate(self):
        self.doc.multiBuild(self.story)

    def confidential(self, canvas):
        canvas.saveState()

        canvas.translate(18.5*cm, 27.4*cm)

        canvas.setLineWidth(3)
        canvas.setFillColorRGB(1, 0, 0)
        canvas.setStrokeGray(0.5)

        p = canvas.beginPath()
        p.moveTo(10, 0)
        p.lineTo(20, 10)
        p.lineTo(30, 0)
        p.lineTo(40, 10)
        p.lineTo(30, 20)
        p.lineTo(40, 30)
        p.lineTo(30, 40)
        p.lineTo(20, 30)
        p.lineTo(10, 40)
        p.lineTo(0, 30)
        p.lineTo(10, 20)
        p.lineTo(0, 10)

        canvas.drawPath(p, fill=1, stroke=0)

        canvas.restoreState()

    def draw_watermark(self, canvas):
        if self._watermark:
            canvas.saveState()
            canvas.rotate(60)
            canvas.setFillColorRGB(0.9, 0.9, 0.9)
            canvas.setFont('%s' % self.style.fontName, 120)
            canvas.drawCentredString(195*mm, -30*mm, self._watermark)
            canvas.restoreState()

    def draw_svg(self, canvas, path, xpos=0, ypos=0, xsize=None, ysize=None):
        from reportlab.graphics import renderPDF
        from svglib.svglib import svg2rlg

        drawing = svg2rlg(path)
        xL, yL, xH, yH = drawing.getBounds()

        if xsize:
            drawing.renderScale = xsize / (xH - xL)
        if ysize:
            drawing.renderScale = ysize / (yH - yL)

        renderPDF.draw(
            drawing, canvas, xpos, ypos, showBoundary=self.show_boundaries)

    def next_frame(self):
        self.story.append(CondPageBreak(20*cm))

    def start_keeptogether(self):
        self.keeptogether_index = len(self.story)

    def end_keeptogether(self):
        keeptogether = KeepTogether(self.story[self.keeptogether_index:])
        self.story = self.story[:self.keeptogether_index]
        self.story.append(keeptogether)

    def address_head(self, text):
        self.smaller(text)
        self.spacer(2*mm)

    def address(self, obj, prefix=''):
        if type(obj) == dict:
            data = obj
        else:
            data = {}
            for field in (
                    'company', 'manner_of_address', 'first_name', 'last_name',
                    'address', 'zip_code', 'city', 'full_override'):
                attribute = '%s%s' % (prefix, field)
                data[field] = getattr(obj, attribute, u'').strip()

        address = []
        if data.get('company', False):
            address.append(data['company'])

        title = data.get('manner_of_address', '')
        if title:
            title += u' '

        if data.get('first_name', False):
            address.append(u'%s%s %s' % (title, data.get('first_name', ''),
                                         data.get('last_name', '')))
        else:
            address.append(u'%s%s' % (title, data.get('last_name', '')))

        address.append(data.get('address'))
        address.append(u'%s %s' % (
            data.get('zip_code', ''),
            data.get('city', '')))

        if data.get('full_override'):
            address = [
                l.strip() for l in
                data.get('full_override').replace('\r', '').splitlines()]

        self.p('\n'.join(address))



# -*-  coding: utf-8 -*-
# Copyright (C) 2015 ZetaOps Inc.
#
# This file is licensed under the GNU General Public License v3
# (GPLv3).  See LICENSE.txt for details.
#
from ulakbus.lib.views import UlakbusView
from zengine.forms import JsonForm
from zengine.forms import fields
from ulakbus.services.zato_wrapper import E_PostaYolla
from zengine.lib.translation import gettext as _, gettext_lazy as __
from ulakbus.lib.common import aktivasyon_kodu_uret
from ulakbus.lib.common import EPostaDogrulama
from ulakbus.lib.common import e_posta_uygunlugu
from ulakbus.settings import DEMO_URL, MAIL_ADDRESS


class EPostaForm(JsonForm):
    birincil_e_posta = fields.String(__(u"Birincil e-postanız"))


class EPostaDegistir(UlakbusView):
    """
    Kullanıcıların birincil e-posta adreslerini değiştirebilmelerini sağlar,
    bu değişim yapılırken kullanıcıdan yeni belirlediği e-posta adresini doğrulaması
    istenir.
    """

    def yeni_e_posta_girisi(self):
        """
        Kullanıcının birincil e_posta değişikliğini yapabileceği ekran oluşturulur
        ve birincil olarak belirlemek istediği e_posta adresini girmesi istenir.
        Bu işlem sonunda girdiği adrese doğrulama linki gönderilecektir.

        """
        if self.current.task_data.get('msg', None):
            self.mesaj_kutusu_goster(_(u'Geçersiz E-Posta Adresi'))

        self.current.task_data['deneme_sayisi'] = 3
        _form = EPostaForm(current=self.current, title=_(u'Yeni E-Posta Girişi'))
        _form.help_text = _(u"""Birincil olarak belirlemek istediğiniz e-posta adresinize
                          doğrulama linki gönderilecektir.""")
        _form.birincil_e_posta = self.current.user.e_mail
        _form.e_posta = fields.String(
            _(u"Birincil olarak belirlemek istediğiniz e-posta adresinizi yazınız."))
        _form.degistir = fields.Button(_(u"Doğrulama Linki Yolla"))
        self.form_out(_form)

    def e_posta_bilgisi_kontrol(self):
        """
        Girilen e-posta adresinin doğruluğu belirlenen kalıpla kontrol edilir.
        """
        self.current.task_data["e_posta"] = self.input['form']['e_posta']
        self.current.task_data['uygunluk'] = e_posta_uygunlugu(self.current.task_data["e_posta"])
        self.current.task_data['msg'] = _(u"""Girmiş olduğunuz e-posta adresi geçersizdir.
                                        Lütfen düzelterek tekrar deneyiniz.""")

    def e_posta_bilgisi_cache_koy_e_posta_hazirla(self):
        """
        Doğrulama linki gönderilecek e_posta adresi, oluşturulan aktivasyon kodu
        ile cache'e kaydedilir. Gönderilecek e-postanın içeriği ve linki wf ismi
        ve aktivasyon kodu ile hazırlanır.
        """
        self.current.task_data['msg'] = None
        self.current.task_data["aktivasyon"] = aktivasyon_kodu_uret()
        EPostaDogrulama(self.current.task_data["aktivasyon"]).set(self.current.task_data["e_posta"],
                                                                  7200)
        self.current.task_data["message"] = "E-Posta adresinizi doğrulamak için aşağıdaki linke" \
                                            " tıklayınız:\n\n %s/#/%s/dogrulama=%s" % (
        DEMO_URL, self.current.task_data['wf_name'], self.current.task_data["aktivasyon"])
        self.current.task_data['subject'] = 'Ulakbüs Aktivasyon Maili'

    def aktivasyon_maili_yolla(self):
        """
        Hashlenmiş 40 karakterli bir aktivasyon kodu üretilir ve cache'e atılır. Zato servisi ile
        kullanıcının yeni olarak belirlediği e_posta adresine doğrulama linki gönderilir.

        """

        posta_gonder = E_PostaYolla(service_payload={
            "default_e_mail": MAIL_ADDRESS,
            "e_posta": self.current.task_data["e_posta"],
            "message": self.current.task_data["message"],
            "subject": self.current.task_data["subject"]})

        posta_gonder.zato_request()

    def link_gonderimi_bilgilendir(self):
        """
        Doğrulama linki yollandığında kullanıcı linkin yollandığına dair bilgilendirilir.

        """
        self.current.task_data['msg'] = _(u"""'%s' adresinize doğrulama linki gönderilmiştir.
        Lütfen e-posta'nızdaki bağlantı linkine tıklayarak e-posta adresinin size ait
        olduğunu doğrulayınız. """) % (self.current.task_data['e_posta'])

        self.mesaj_kutusu_goster('E-Posta Doğrulama', 'info')



# -*- coding: utf-8 -*-

# Copyright (C) 2015 ZetaOps Inc.
#
# This file is licensed under the GNU General Public License v3
# (GPLv3).  See LICENSE.txt for details.


__author__ = 'Ali Riza Keles'

import json
from ulakbus.services.ulakbus_service import UlakbusService


class GetIPAddress(UlakbusService):
    """
    Informative service for development purpose
    Returns IP address of zato server's public interface
    """

    HAS_CHANNEL = True

    def handle(self):
        service = self.outgoing.plain_http.get('IPIFY')
        response = service.conn.get(self.cid)
        self.response.payload = {"status": "ok", "result": json.dumps(response.data)}
